@@ -69,16 +69,12 @@ impl Engine {
         let typ = self.map_type_name(result.type_name());
 
         result.try_cast().ok_or_else(|| {
-            ERR::ErrorMismatchOutputType(
-                self.map_type_name(type_name::<T>()).into(),
-                typ.into(),
-                Position::NONE,
-            )
-            .into()
+            let t = self.map_type_name(type_name::<T>()).into();
+            ERR::ErrorMismatchOutputType(t, typ.into(), Position::NONE).into()
         })
     }
-    /// Call a script function defined in an [`AST`] with multiple [`Dynamic`] arguments
-    /// and the following options:
+    /// Call a script function defined in an [`AST`] with multiple [`Dynamic`] arguments and the
+    /// following options:
     ///
     /// * whether to evaluate the [`AST`] to load necessary modules before calling the function
     /// * whether to rewind the [`Scope`] after the function call
@@ -95,8 +91,8 @@ impl Engine {
     /// All the arguments are _consumed_, meaning that they're replaced by `()`.
     /// This is to avoid unnecessarily cloning the arguments.
     ///
-    /// Do not use the arguments after this call. If they are needed afterwards,
-    /// clone them _before_ calling this function.
+    /// Do not use the arguments after this call. If they are needed afterwards, clone them _before_
+    /// calling this function.
     ///
     /// # Example
     ///
@@ -154,7 +150,8 @@ impl Engine {
         arg_values: impl AsMut<[Dynamic]>,
     ) -> RhaiResult {
         let state = &mut EvalState::new();
-        let global = &mut GlobalRuntimeState::new();
+        let global = &mut GlobalRuntimeState::new(self);
+
         let statements = ast.statements();
 
         let orig_scope_len = scope.len();
@@ -181,7 +178,7 @@ impl Engine {
         #[cfg(not(feature = "no_closure"))]
         crate::func::call::ensure_no_data_race(name, &mut args, false)?;
 
-        let result = self.call_script_fn(
+        self.call_script_fn(
             scope,
             global,
             state,
@@ -189,11 +186,9 @@ impl Engine {
             &mut this_ptr,
             fn_def,
             &mut args,
-            Position::NONE,
             rewind_scope,
+            Position::NONE,
             0,
-        );
-
-        result
+        )
     }
 }
