@@ -5,6 +5,19 @@ use crate::{Dynamic, Engine, EvalContext, Position, RhaiResultOf};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 
+/// Information on a variable definition.
+#[non_exhaustive]
+pub struct VarDefInfo<'a> {
+    /// Name of the variable to be defined.
+    pub name: &'a str,
+    /// `true` if the statement is `const`, otherwise it is `let`.
+    pub is_const: bool,
+    /// The current nesting level, with zero being the global level.
+    pub nesting_level: usize,
+    /// Will the variable _shadow_ an existing variable?
+    pub will_shadow: bool,
+}
+
 impl Engine {
     /// Provide a callback that will be invoked before each variable access.
     ///
@@ -12,16 +25,15 @@ impl Engine {
     ///
     /// The callback function signature takes the following form:
     ///
-    /// > `Fn(name: &str, index: usize, context: &EvalContext)`  
-    /// > `   -> Result<Option<Dynamic>, Box<EvalAltResult>> + 'static`
+    /// > `Fn(name: &str, index: usize, context: &EvalContext) -> Result<Option<Dynamic>, Box<EvalAltResult>>`
     ///
     /// where:
+    /// * `name`: name of the variable.
     /// * `index`: an offset from the bottom of the current [`Scope`][crate::Scope] that the
     ///   variable is supposed to reside. Offsets start from 1, with 1 meaning the last variable in
     ///   the current [`Scope`][crate::Scope].  Essentially the correct variable is at position
     ///   `scope.len() - index`. If `index` is zero, then there is no pre-calculated offset position
     ///   and a search through the current [`Scope`][crate::Scope] must be performed.
-    ///
     /// * `context`: the current [evaluation context][`EvalContext`].
     ///
     /// ## Return value
@@ -64,7 +76,69 @@ impl Engine {
         self.resolve_var = Some(Box::new(callback));
         self
     }
-    /// _(internals)_ Provide a callback that will be invoked during parsing to remap certain tokens.
+    /// Provide a callback that will be invoked before the definition of each variable .
+    ///
+    /// # WARNING - Unstable API
+    ///
+    /// This API is volatile and may change in the future.
+    ///
+    /// # Callback Function Signature
+    ///
+    /// The callback function signature takes the following form:
+    ///
+    /// > `Fn(is_runtime: bool, info: VarInfo, context: &EvalContext) -> Result<bool, Box<EvalAltResult>>`
+    ///
+    /// where:
+    /// * `is_runtime`: `true` if the variable definition event happens during runtime, `false` if during compilation.
+    /// * `info`: information on the variable.
+    /// * `context`: the current [evaluation context][`EvalContext`].
+    ///
+    /// ## Return value
+    ///
+    /// * `Ok(true)`: continue with normal variable definition.
+    /// * `Ok(false)`: deny the variable definition with an [runtime error][crate::EvalAltResult::ErrorRuntime].
+    ///
+    /// ## Raising errors
+    ///
+    /// Return `Err(...)` if there is an error.
+    ///
+    /// # Example
+    ///
+    /// ```should_panic
+    /// # fn main() -> Result<(), Box<rhai::EvalAltResult>> {
+    /// use rhai::Engine;
+    ///
+    /// let mut engine = Engine::new();
+    ///
+    /// // Register a variable definition filter.
+    /// engine.on_def_var(|_, info, _| {
+    ///     // Disallow defining MYSTIC_NUMBER as a constant
+    ///     if info.name == "MYSTIC_NUMBER" && info.is_const {
+    ///         Ok(false)
+    ///     } else {
+    ///         Ok(true)
+    ///     }
+    /// });
+    ///
+    /// // The following runs fine:
+    /// engine.eval::<i64>("let MYSTIC_NUMBER = 42;")?;
+    ///
+    /// // The following will cause an error:
+    /// engine.eval::<i64>("const MYSTIC_NUMBER = 42;")?;
+    ///
+    /// # Ok(())
+    /// # }
+    /// ```
+    #[deprecated = "This API is volatile and may change in the future."]
+    #[inline(always)]
+    pub fn on_def_var(
+        &mut self,
+        callback: impl Fn(bool, VarDefInfo, &EvalContext) -> RhaiResultOf<bool> + SendSync + 'static,
+    ) -> &mut Self {
+        self.def_var_filter = Some(Box::new(callback));
+        self
+    }
+    /// _(internals)_ Register a callback that will be invoked during parsing to remap certain tokens.
     /// Exported under the `internals` feature only.
     ///
     /// # Callback Function Signature
@@ -208,7 +282,7 @@ impl Engine {
     /// ```
     #[inline(always)]
     pub fn on_print(&mut self, callback: impl Fn(&str) + SendSync + 'static) -> &mut Self {
-        self.print = Some(Box::new(callback));
+        self.print = Box::new(callback);
         self
     }
     /// Override default action of `debug` (print to stdout using [`println!`])
@@ -258,7 +332,32 @@ impl Engine {
         &mut self,
         callback: impl Fn(&str, Option<&str>, Position) + SendSync + 'static,
     ) -> &mut Self {
-        self.debug = Some(Box::new(callback));
+        self.debug = Box::new(callback);
+        self
+    }
+    /// _(debugging)_ Register a callback for debugging.
+    /// Exported under the `debugging` feature only.
+    ///
+    /// # WARNING - Unstable API
+    ///
+    /// This API is volatile and may change in the future.
+    #[deprecated = "This API is volatile and may change in the future."]
+    #[cfg(feature = "debugging")]
+    #[inline(always)]
+    pub fn register_debugger(
+        &mut self,
+        init: impl Fn() -> Dynamic + SendSync + 'static,
+        callback: impl Fn(
+                &mut EvalContext,
+                crate::eval::DebuggerEvent,
+                crate::ast::ASTNode,
+                Option<&str>,
+                Position,
+            ) -> RhaiResultOf<crate::eval::DebuggerCommand>
+            + SendSync
+            + 'static,
+    ) -> &mut Self {
+        self.debugger = Some((Box::new(init), Box::new(callback)));
         self
     }
 }
