@@ -10,7 +10,7 @@ use std::any::{type_name, TypeId};
 use std::prelude::v1::*;
 
 impl Engine {
-    /// Get the global namespace module (which is the last module in `global_modules`).
+    /// Get the global namespace module (which is the fist module in `global_modules`).
     #[inline(always)]
     #[allow(dead_code)]
     pub(crate) fn global_namespace(&self) -> &Module {
@@ -273,7 +273,8 @@ impl Engine {
     /// ```
     #[inline(always)]
     pub fn register_type_with_name<T: Variant + Clone>(&mut self, name: &str) -> &mut Self {
-        self.register_type_with_name_raw(type_name::<T>(), name)
+        self.global_namespace_mut().set_custom_type::<T>(name);
+        self
     }
     /// Register a custom type for use with the [`Engine`], with a pretty-print name
     /// for the `type_of` function. The type must implement [`Clone`].
@@ -288,12 +289,12 @@ impl Engine {
         name: impl Into<Identifier>,
     ) -> &mut Self {
         // Add the pretty-print type name into the map
-        self.type_names
-            .insert(fully_qualified_type_path.into(), name.into());
+        self.global_namespace_mut()
+            .set_custom_type_raw(fully_qualified_type_path, name);
         self
     }
     /// Register an type iterator for an iterable type with the [`Engine`].
-    /// This is an advanced feature.
+    /// This is an advanced API.
     #[inline(always)]
     pub fn register_iterator<T>(&mut self) -> &mut Self
     where
@@ -1003,7 +1004,7 @@ impl Engine {
                 let sub_module = iter.next().expect("contains separator").trim();
                 let remainder = iter.next().expect("contains separator").trim();
 
-                if !root.contains_key(sub_module) {
+                if root.is_empty() || !root.contains_key(sub_module) {
                     let mut m = Module::new();
                     register_static_module_raw(m.sub_modules_mut(), remainder, module);
                     m.build_index();
@@ -1037,9 +1038,10 @@ impl Engine {
 
         signatures.extend(self.global_namespace().gen_fn_signatures());
 
-        self.global_sub_modules.iter().for_each(|(name, m)| {
+        #[cfg(not(feature = "no_module"))]
+        for (name, m) in &self.global_sub_modules {
             signatures.extend(m.gen_fn_signatures().map(|f| format!("{}::{}", name, f)))
-        });
+        }
 
         signatures.extend(
             self.global_modules
