@@ -1,8 +1,8 @@
 //! Module defining script statements.
 
-use super::{ASTNode, Expr, FnCallExpr, Ident, OptionFlags, AST_OPTION_FLAGS};
+use super::{ASTFlags, ASTNode, BinaryExpr, Expr, FnCallExpr, Ident};
 use crate::engine::KEYWORD_EVAL;
-use crate::tokenizer::Token;
+use crate::tokenizer::{Span, Token};
 use crate::{calc_fn_hash, Position, StaticVec, INT};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
@@ -11,6 +11,7 @@ use std::{
     fmt,
     hash::Hash,
     mem,
+    num::NonZeroUsize,
     ops::{Deref, DerefMut},
 };
 
@@ -23,6 +24,8 @@ pub struct OpAssignment<'a> {
     /// Hash of the underlying operator call (for fallback).
     pub hash_op: u64,
     /// Op-assignment operator.
+    pub op_assign: &'a str,
+    /// Underlying operator.
     pub op: &'a str,
 }
 
@@ -51,7 +54,8 @@ impl OpAssignment<'_> {
         Self {
             hash_op_assign: calc_fn_hash(op.literal_syntax(), 2),
             hash_op: calc_fn_hash(op_raw, 2),
-            op: op.literal_syntax(),
+            op_assign: op.literal_syntax(),
+            op: op_raw,
         }
     }
     /// Create a new [`OpAssignment`] from a base operator.
@@ -76,27 +80,123 @@ impl OpAssignment<'_> {
     }
 }
 
+/// A statements block with an optional condition.
+#[derive(Debug, Clone, Hash)]
+pub struct ConditionalStmtBlock {
+    /// Optional condition.
+    pub condition: Option<Expr>,
+    /// Statements block.
+    pub statements: StmtBlock,
+}
+
+impl<B: Into<StmtBlock>> From<B> for ConditionalStmtBlock {
+    #[inline(always)]
+    fn from(value: B) -> Self {
+        Self {
+            condition: None,
+            statements: value.into(),
+        }
+    }
+}
+
+impl<B: Into<StmtBlock>> From<(Expr, B)> for ConditionalStmtBlock {
+    #[inline(always)]
+    fn from(value: (Expr, B)) -> Self {
+        Self {
+            condition: Some(value.0),
+            statements: value.1.into(),
+        }
+    }
+}
+
+impl<B: Into<StmtBlock>> From<(Option<Expr>, B)> for ConditionalStmtBlock {
+    #[inline(always)]
+    fn from(value: (Option<Expr>, B)) -> Self {
+        Self {
+            condition: value.0,
+            statements: value.1.into(),
+        }
+    }
+}
+
+impl ConditionalStmtBlock {
+    /// Does the condition exist?
+    #[inline(always)]
+    #[must_use]
+    pub const fn has_condition(&self) -> bool {
+        self.condition.is_some()
+    }
+}
+
+/// _(internals)_ A type containing all cases for a `switch` statement.
+/// Exported under the `internals` feature only.
+#[derive(Debug, Clone, Hash)]
+pub struct SwitchCases {
+    /// Dictionary mapping value hashes to [`ConditionalStmtBlock`]'s.
+    pub cases: BTreeMap<u64, Box<ConditionalStmtBlock>>,
+    /// Statements block for the default case (there can be no condition for the default case).
+    pub def_case: Box<StmtBlock>,
+    /// List of range cases.
+    pub ranges: StaticVec<(INT, INT, bool, Box<ConditionalStmtBlock>)>,
+}
+
+/// _(internals)_ A `try-catch` block.
+/// Exported under the `internals` feature only.
+#[derive(Debug, Clone, Hash)]
+pub struct TryCatchBlock {
+    /// `try` block.
+    pub try_block: StmtBlock,
+    /// `catch` variable, if any.
+    pub catch_var: Ident,
+    /// `catch` block.
+    pub catch_block: StmtBlock,
+}
+
+/// _(internals)_ The underlying container type for [`StmtBlock`].
+/// Exported under the `internals` feature only.
+///
+/// A [`SmallVec`](https://crates.io/crates/smallvec) containing up to 8 items inline is used to
+/// hold a statements block, with the assumption that most program blocks would container fewer than
+/// 8 statements, and those that do have a lot more statements.
+#[cfg(not(feature = "no_std"))]
+pub type StmtBlockContainer = smallvec::SmallVec<[Stmt; 8]>;
+
+/// _(internals)_ The underlying container type for [`StmtBlock`].
+/// Exported under the `internals` feature only.
+#[cfg(feature = "no_std")]
+pub type StmtBlockContainer = StaticVec<Stmt>;
+
 /// _(internals)_ A scoped block of statements.
 /// Exported under the `internals` feature only.
 #[derive(Clone, Hash, Default)]
-pub struct StmtBlock(StaticVec<Stmt>, Position);
+pub struct StmtBlock(StmtBlockContainer, Span);
 
 impl StmtBlock {
     /// A [`StmtBlock`] that does not exist.
     pub const NONE: Self = Self::empty(Position::NONE);
 
     /// Create a new [`StmtBlock`].
+    #[inline(always)]
     #[must_use]
-    pub fn new(statements: impl IntoIterator<Item = Stmt>, pos: Position) -> Self {
-        let mut statements: StaticVec<_> = statements.into_iter().collect();
+    pub fn new(
+        statements: impl IntoIterator<Item = Stmt>,
+        start_pos: Position,
+        end_pos: Position,
+    ) -> Self {
+        Self::new_with_span(statements, Span::new(start_pos, end_pos))
+    }
+    /// Create a new [`StmtBlock`].
+    #[must_use]
+    pub fn new_with_span(statements: impl IntoIterator<Item = Stmt>, span: Span) -> Self {
+        let mut statements: smallvec::SmallVec<_> = statements.into_iter().collect();
         statements.shrink_to_fit();
-        Self(statements, pos)
+        Self(statements, span)
     }
     /// Create an empty [`StmtBlock`].
     #[inline(always)]
     #[must_use]
     pub const fn empty(pos: Position) -> Self {
-        Self(StaticVec::new_const(), pos)
+        Self(StmtBlockContainer::new_const(), Span::new(pos, pos))
     }
     /// Is this statements block empty?
     #[inline(always)]
@@ -119,7 +219,7 @@ impl StmtBlock {
     /// Extract the statements.
     #[inline(always)]
     #[must_use]
-    pub(crate) fn take_statements(&mut self) -> StaticVec<Stmt> {
+    pub(crate) fn take_statements(&mut self) -> StmtBlockContainer {
         mem::take(&mut self.0)
     }
     /// Get an iterator over the statements of this statements block.
@@ -128,21 +228,43 @@ impl StmtBlock {
     pub fn iter(&self) -> impl Iterator<Item = &Stmt> {
         self.0.iter()
     }
-    /// Get the position (location of the beginning `{`) of this statements block.
+    /// Get the start position (location of the beginning `{`) of this statements block.
     #[inline(always)]
     #[must_use]
     pub const fn position(&self) -> Position {
+        (self.1).start()
+    }
+    /// Get the end position (location of the ending `}`) of this statements block.
+    #[inline(always)]
+    #[must_use]
+    pub const fn end_position(&self) -> Position {
+        (self.1).end()
+    }
+    /// Get the positions (locations of the beginning `{` and ending `}`) of this statements block.
+    #[inline(always)]
+    #[must_use]
+    pub const fn span(&self) -> Span {
         self.1
     }
-    /// Set the position (location of the beginning `{`) of this statements block.
+    /// Get the positions (locations of the beginning `{` and ending `}`) of this statements block
+    /// or a default.
     #[inline(always)]
-    pub fn set_position(&mut self, pos: Position) {
-        self.1 = pos;
+    #[must_use]
+    pub const fn span_or_else(&self, def_start_pos: Position, def_end_pos: Position) -> Span {
+        Span::new(
+            (self.1).start().or_else(def_start_pos),
+            (self.1).end().or_else(def_end_pos),
+        )
+    }
+    /// Set the positions of this statements block.
+    #[inline(always)]
+    pub fn set_position(&mut self, start_pos: Position, end_pos: Position) {
+        self.1 = Span::new(start_pos, end_pos);
     }
 }
 
 impl Deref for StmtBlock {
-    type Target = StaticVec<Stmt>;
+    type Target = StmtBlockContainer;
 
     #[inline(always)]
     fn deref(&self) -> &Self::Target {
@@ -157,11 +279,28 @@ impl DerefMut for StmtBlock {
     }
 }
 
+impl AsRef<[Stmt]> for StmtBlock {
+    #[inline(always)]
+    fn as_ref(&self) -> &[Stmt] {
+        &self.0
+    }
+}
+
+impl AsMut<[Stmt]> for StmtBlock {
+    #[inline(always)]
+    fn as_mut(&mut self) -> &mut [Stmt] {
+        &mut self.0
+    }
+}
+
 impl fmt::Debug for StmtBlock {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("Block")?;
         fmt::Debug::fmt(&self.0, f)?;
-        self.1.debug_print(f)
+        if !self.1.is_none() {
+            write!(f, " @ {:?}", self.1)?;
+        }
+        Ok(())
     }
 }
 
@@ -169,11 +308,11 @@ impl From<Stmt> for StmtBlock {
     #[inline]
     fn from(stmt: Stmt) -> Self {
         match stmt {
-            Stmt::Block(mut block, pos) => Self(block.iter_mut().map(mem::take).collect(), pos),
-            Stmt::Noop(pos) => Self(StaticVec::new_const(), pos),
+            Stmt::Block(block) => *block,
+            Stmt::Noop(pos) => Self(StmtBlockContainer::new_const(), Span::new(pos, pos)),
             _ => {
                 let pos = stmt.position();
-                Self(vec![stmt].into(), pos)
+                Self(vec![stmt].into(), Span::new(pos, Position::NONE))
             }
         }
     }
@@ -181,6 +320,9 @@ impl From<Stmt> for StmtBlock {
 
 impl IntoIterator for StmtBlock {
     type Item = Stmt;
+    #[cfg(not(feature = "no_std"))]
+    type IntoIter = smallvec::IntoIter<[Stmt; 8]>;
+    #[cfg(feature = "no_std")]
     type IntoIter = smallvec::IntoIter<[Stmt; 3]>;
 
     #[inline(always)]
@@ -203,7 +345,7 @@ pub enum Stmt {
     /// No-op.
     Noop(Position),
     /// `if` expr `{` stmt `}` `else` `{` stmt `}`
-    If(Expr, Box<(StmtBlock, StmtBlock)>, Position),
+    If(Box<(Expr, StmtBlock, StmtBlock)>, Position),
     /// `switch` expr `{` literal or range or _ `if` condition `=>` stmt `,` ... `}`
     ///
     /// ### Data Structure
@@ -211,72 +353,64 @@ pub enum Stmt {
     /// 0) Hash table for (condition, block)
     /// 1) Default block
     /// 2) List of ranges: (start, end, inclusive, condition, statement)
-    Switch(
-        Expr,
-        Box<(
-            BTreeMap<u64, Box<(Option<Expr>, StmtBlock)>>,
-            StmtBlock,
-            StaticVec<(INT, INT, bool, Option<Expr>, StmtBlock)>,
-        )>,
-        Position,
-    ),
+    Switch(Box<(Expr, SwitchCases)>, Position),
     /// `while` expr `{` stmt `}` | `loop` `{` stmt `}`
     ///
     /// If the guard expression is [`UNIT`][Expr::Unit], then it is a `loop` statement.
-    While(Expr, Box<StmtBlock>, Position),
+    While(Box<(Expr, StmtBlock)>, Position),
     /// `do` `{` stmt `}` `while`|`until` expr
     ///
-    /// ### Option Flags
+    /// ### Flags
     ///
-    /// * [`AST_OPTION_NONE`][AST_OPTION_FLAGS::AST_OPTION_NONE] = `while`
-    /// * [`AST_OPTION_NEGATED`][AST_OPTION_FLAGS::AST_OPTION_NEGATED] = `until`
-    Do(Box<StmtBlock>, Expr, OptionFlags, Position),
+    /// * [`NONE`][ASTFlags::NONE] = `while`
+    /// * [`NEGATED`][ASTFlags::NEGATED] = `until`
+    Do(Box<(Expr, StmtBlock)>, ASTFlags, Position),
     /// `for` `(` id `,` counter `)` `in` expr `{` stmt `}`
-    For(Expr, Box<(Ident, Option<Ident>, StmtBlock)>, Position),
+    For(Box<(Ident, Ident, Expr, StmtBlock)>, Position),
     /// \[`export`\] `let`|`const` id `=` expr
     ///
-    /// ### Option Flags
+    /// ### Flags
     ///
-    /// * [`AST_OPTION_PUBLIC`][AST_OPTION_FLAGS::AST_OPTION_PUBLIC] = `export`
-    /// * [`AST_OPTION_CONSTANT`][AST_OPTION_FLAGS::AST_OPTION_CONSTANT] = `const`
-    Var(Expr, Box<Ident>, OptionFlags, Position),
+    /// * [`EXPORTED`][ASTFlags::EXPORTED] = `export`
+    /// * [`CONSTANT`][ASTFlags::CONSTANT] = `const`
+    Var(Box<(Ident, Expr, Option<NonZeroUsize>)>, ASTFlags, Position),
     /// expr op`=` expr
-    Assignment(Box<(Expr, Option<OpAssignment<'static>>, Expr)>, Position),
+    Assignment(Box<(Option<OpAssignment<'static>>, BinaryExpr)>, Position),
     /// func `(` expr `,` ... `)`
     ///
     /// Note - this is a duplicate of [`Expr::FnCall`] to cover the very common pattern of a single
     ///        function call forming one statement.
     FnCall(Box<FnCallExpr>, Position),
     /// `{` stmt`;` ... `}`
-    Block(Box<[Stmt]>, Position),
+    Block(Box<StmtBlock>),
     /// `try` `{` stmt; ... `}` `catch` `(` var `)` `{` stmt; ... `}`
-    TryCatch(Box<(StmtBlock, Option<Ident>, StmtBlock)>, Position),
+    TryCatch(Box<TryCatchBlock>, Position),
     /// [expression][Expr]
-    Expr(Expr),
+    Expr(Box<Expr>),
     /// `continue`/`break`
     ///
-    /// ### Option Flags
+    /// ### Flags
     ///
-    /// * [`AST_OPTION_NONE`][AST_OPTION_FLAGS::AST_OPTION_NONE] = `continue`
-    /// * [`AST_OPTION_BREAK_OUT`][AST_OPTION_FLAGS::AST_OPTION_BREAK_OUT] = `break`
-    BreakLoop(OptionFlags, Position),
+    /// * [`NONE`][ASTFlags::NONE] = `continue`
+    /// * [`BREAK`][ASTFlags::BREAK] = `break`
+    BreakLoop(ASTFlags, Position),
     /// `return`/`throw`
     ///
-    /// ### Option Flags
+    /// ### Flags
     ///
-    /// * [`AST_OPTION_NONE`][AST_OPTION_FLAGS::AST_OPTION_NONE] = `return`
-    /// * [`AST_OPTION_BREAK_OUT`][AST_OPTION_FLAGS::AST_OPTION_BREAK_OUT] = `throw`
-    Return(OptionFlags, Option<Expr>, Position),
-    /// `import` expr `as` var
+    /// * [`NONE`][ASTFlags::NONE] = `return`
+    /// * [`BREAK`][ASTFlags::BREAK] = `throw`
+    Return(Option<Box<Expr>>, ASTFlags, Position),
+    /// `import` expr `as` alias
     ///
     /// Not available under `no_module`.
     #[cfg(not(feature = "no_module"))]
-    Import(Expr, Option<Box<Ident>>, Position),
-    /// `export` var `as` var `,` ...
+    Import(Box<(Expr, Ident)>, Position),
+    /// `export` var `as` alias
     ///
     /// Not available under `no_module`.
     #[cfg(not(feature = "no_module"))]
-    Export(Box<[(Ident, Ident)]>, Position),
+    Export(Box<(Ident, Ident)>, Position),
     /// Convert a variable to shared.
     ///
     /// Not available under `no_closure`.
@@ -286,7 +420,7 @@ pub enum Stmt {
     /// This variant does not map to any language structure.  It is currently only used only to
     /// convert a normal variable into a shared variable when the variable is _captured_ by a closure.
     #[cfg(not(feature = "no_closure"))]
-    Share(crate::Identifier),
+    Share(Box<crate::Identifier>, Position),
 }
 
 impl Default for Stmt {
@@ -299,7 +433,21 @@ impl Default for Stmt {
 impl From<StmtBlock> for Stmt {
     #[inline(always)]
     fn from(block: StmtBlock) -> Self {
-        Self::Block(block.0.into_boxed_slice(), block.1)
+        Self::Block(block.into())
+    }
+}
+
+impl<T: IntoIterator<Item = Stmt>> From<(T, Position, Position)> for Stmt {
+    #[inline(always)]
+    fn from(value: (T, Position, Position)) -> Self {
+        StmtBlock::new(value.0, value.1, value.2).into()
+    }
+}
+
+impl<T: IntoIterator<Item = Stmt>> From<(T, Span)> for Stmt {
+    #[inline(always)]
+    fn from(value: (T, Span)) -> Self {
+        StmtBlock::new_with_span(value.0, value.1).into()
     }
 }
 
@@ -308,65 +456,67 @@ impl Stmt {
     #[inline(always)]
     #[must_use]
     pub const fn is_noop(&self) -> bool {
-        matches!(self, Self::Noop(_))
+        matches!(self, Self::Noop(..))
     }
     /// Get the [position][Position] of this statement.
     #[must_use]
-    pub const fn position(&self) -> Position {
+    pub fn position(&self) -> Position {
         match self {
             Self::Noop(pos)
-            | Self::BreakLoop(_, pos)
-            | Self::Block(_, pos)
-            | Self::Assignment(_, pos)
-            | Self::FnCall(_, pos)
-            | Self::If(_, _, pos)
-            | Self::Switch(_, _, pos)
-            | Self::While(_, _, pos)
-            | Self::Do(_, _, _, pos)
-            | Self::For(_, _, pos)
-            | Self::Return(_, _, pos)
-            | Self::Var(_, _, _, pos)
-            | Self::TryCatch(_, pos) => *pos,
+            | Self::BreakLoop(.., pos)
+            | Self::Assignment(.., pos)
+            | Self::FnCall(.., pos)
+            | Self::If(.., pos)
+            | Self::Switch(.., pos)
+            | Self::While(.., pos)
+            | Self::Do(.., pos)
+            | Self::For(.., pos)
+            | Self::Return(.., pos)
+            | Self::Var(.., pos)
+            | Self::TryCatch(.., pos) => *pos,
 
-            Self::Expr(x) => x.position(),
+            Self::Block(x) => x.position(),
+
+            Self::Expr(x) => x.start_position(),
 
             #[cfg(not(feature = "no_module"))]
-            Self::Import(_, _, pos) => *pos,
+            Self::Import(.., pos) => *pos,
             #[cfg(not(feature = "no_module"))]
-            Self::Export(_, pos) => *pos,
+            Self::Export(.., pos) => *pos,
 
             #[cfg(not(feature = "no_closure"))]
-            Self::Share(_) => Position::NONE,
+            Self::Share(.., pos) => *pos,
         }
     }
     /// Override the [position][Position] of this statement.
     pub fn set_position(&mut self, new_pos: Position) -> &mut Self {
         match self {
             Self::Noop(pos)
-            | Self::BreakLoop(_, pos)
-            | Self::Block(_, pos)
-            | Self::Assignment(_, pos)
-            | Self::FnCall(_, pos)
-            | Self::If(_, _, pos)
-            | Self::Switch(_, _, pos)
-            | Self::While(_, _, pos)
-            | Self::Do(_, _, _, pos)
-            | Self::For(_, _, pos)
-            | Self::Return(_, _, pos)
-            | Self::Var(_, _, _, pos)
-            | Self::TryCatch(_, pos) => *pos = new_pos,
+            | Self::BreakLoop(.., pos)
+            | Self::Assignment(.., pos)
+            | Self::FnCall(.., pos)
+            | Self::If(.., pos)
+            | Self::Switch(.., pos)
+            | Self::While(.., pos)
+            | Self::Do(.., pos)
+            | Self::For(.., pos)
+            | Self::Return(.., pos)
+            | Self::Var(.., pos)
+            | Self::TryCatch(.., pos) => *pos = new_pos,
+
+            Self::Block(x) => x.set_position(new_pos, x.end_position()),
 
             Self::Expr(x) => {
                 x.set_position(new_pos);
             }
 
             #[cfg(not(feature = "no_module"))]
-            Self::Import(_, _, pos) => *pos = new_pos,
+            Self::Import(.., pos) => *pos = new_pos,
             #[cfg(not(feature = "no_module"))]
-            Self::Export(_, pos) => *pos = new_pos,
+            Self::Export(.., pos) => *pos = new_pos,
 
             #[cfg(not(feature = "no_closure"))]
-            Self::Share(_) => (),
+            Self::Share(.., pos) => *pos = new_pos,
         }
 
         self
@@ -375,59 +525,58 @@ impl Stmt {
     #[must_use]
     pub const fn returns_value(&self) -> bool {
         match self {
-            Self::If(_, _, _)
-            | Self::Switch(_, _, _)
-            | Self::Block(_, _)
-            | Self::Expr(_)
-            | Self::FnCall(_, _) => true,
+            Self::If(..)
+            | Self::Switch(..)
+            | Self::Block(..)
+            | Self::Expr(..)
+            | Self::FnCall(..) => true,
 
-            Self::Noop(_)
-            | Self::While(_, _, _)
-            | Self::Do(_, _, _, _)
-            | Self::For(_, _, _)
-            | Self::TryCatch(_, _) => false,
+            Self::Noop(..)
+            | Self::While(..)
+            | Self::Do(..)
+            | Self::For(..)
+            | Self::TryCatch(..) => false,
 
-            Self::Var(_, _, _, _)
-            | Self::Assignment(_, _)
-            | Self::BreakLoop(_, _)
-            | Self::Return(_, _, _) => false,
+            Self::Var(..) | Self::Assignment(..) | Self::BreakLoop(..) | Self::Return(..) => false,
 
             #[cfg(not(feature = "no_module"))]
-            Self::Import(_, _, _) | Self::Export(_, _) => false,
+            Self::Import(..) | Self::Export(..) => false,
 
             #[cfg(not(feature = "no_closure"))]
-            Self::Share(_) => false,
+            Self::Share(..) => false,
         }
     }
     /// Is this statement self-terminated (i.e. no need for a semicolon terminator)?
     #[must_use]
     pub const fn is_self_terminated(&self) -> bool {
         match self {
-            Self::If(_, _, _)
-            | Self::Switch(_, _, _)
-            | Self::While(_, _, _)
-            | Self::For(_, _, _)
-            | Self::Block(_, _)
-            | Self::TryCatch(_, _) => true,
+            Self::If(..)
+            | Self::Switch(..)
+            | Self::While(..)
+            | Self::For(..)
+            | Self::Block(..)
+            | Self::TryCatch(..) => true,
 
             // A No-op requires a semicolon in order to know it is an empty statement!
-            Self::Noop(_) => false,
+            Self::Noop(..) => false,
 
-            Self::Expr(Expr::Custom(x, _)) if x.is_self_terminated() => true,
+            Self::Expr(e) => match &**e {
+                Expr::Custom(x, ..) if x.is_self_terminated() => true,
+                _ => false,
+            },
 
-            Self::Var(_, _, _, _)
-            | Self::Assignment(_, _)
-            | Self::Expr(_)
-            | Self::FnCall(_, _)
-            | Self::Do(_, _, _, _)
-            | Self::BreakLoop(_, _)
-            | Self::Return(_, _, _) => false,
+            Self::Var(..)
+            | Self::Assignment(..)
+            | Self::FnCall(..)
+            | Self::Do(..)
+            | Self::BreakLoop(..)
+            | Self::Return(..) => false,
 
             #[cfg(not(feature = "no_module"))]
-            Self::Import(_, _, _) | Self::Export(_, _) => false,
+            Self::Import(..) | Self::Export(..) => false,
 
             #[cfg(not(feature = "no_closure"))]
-            Self::Share(_) => false,
+            Self::Share(..) => false,
         }
     }
     /// Is this statement _pure_?
@@ -436,78 +585,79 @@ impl Stmt {
     #[must_use]
     pub fn is_pure(&self) -> bool {
         match self {
-            Self::Noop(_) => true,
+            Self::Noop(..) => true,
             Self::Expr(expr) => expr.is_pure(),
-            Self::If(condition, x, _) => {
-                condition.is_pure()
-                    && (x.0).0.iter().all(Stmt::is_pure)
-                    && (x.1).0.iter().all(Stmt::is_pure)
+            Self::If(x, ..) => {
+                x.0.is_pure() && x.1.iter().all(Stmt::is_pure) && x.2.iter().all(Stmt::is_pure)
             }
-            Self::Switch(expr, x, _) => {
-                expr.is_pure()
-                    && x.0.values().all(|block| {
-                        block.0.as_ref().map(Expr::is_pure).unwrap_or(true)
-                            && (block.1).0.iter().all(Stmt::is_pure)
+            Self::Switch(x, ..) => {
+                x.0.is_pure()
+                    && x.1.cases.values().all(|block| {
+                        block.condition.as_ref().map(Expr::is_pure).unwrap_or(true)
+                            && block.statements.iter().all(Stmt::is_pure)
                     })
-                    && (x.2).iter().all(|(_, _, _, condition, stmt)| {
-                        condition.as_ref().map(Expr::is_pure).unwrap_or(true)
-                            && stmt.0.iter().all(Stmt::is_pure)
+                    && x.1.ranges.iter().all(|(.., block)| {
+                        block.condition.as_ref().map(Expr::is_pure).unwrap_or(true)
+                            && block.statements.iter().all(Stmt::is_pure)
                     })
-                    && (x.1).0.iter().all(Stmt::is_pure)
+                    && x.1.def_case.iter().all(Stmt::is_pure)
             }
 
             // Loops that exit can be pure because it can never be infinite.
-            Self::While(Expr::BoolConstant(false, _), _, _) => true,
-            Self::Do(body, Expr::BoolConstant(x, _), options, _)
-                if *x == options.contains(AST_OPTION_FLAGS::AST_OPTION_NEGATED) =>
-            {
-                body.iter().all(Stmt::is_pure)
-            }
+            Self::While(x, ..) if matches!(x.0, Expr::BoolConstant(false, ..)) => true,
+            Self::Do(x, options, ..) if matches!(x.0, Expr::BoolConstant(..)) => match x.0 {
+                Expr::BoolConstant(cond, ..) if cond == options.contains(ASTFlags::NEGATED) => {
+                    x.1.iter().all(Stmt::is_pure)
+                }
+                _ => false,
+            },
 
             // Loops are never pure since they can be infinite - and that's a side effect.
-            Self::While(_, _, _) | Self::Do(_, _, _, _) => false,
+            Self::While(..) | Self::Do(..) => false,
 
             // For loops can be pure because if the iterable is pure, it is finite,
             // so infinite loops can never occur.
-            Self::For(iterable, x, _) => iterable.is_pure() && (x.2).0.iter().all(Stmt::is_pure),
+            Self::For(x, ..) => x.2.is_pure() && x.3.iter().all(Stmt::is_pure),
 
-            Self::Var(_, _, _, _) | Self::Assignment(_, _) | Self::FnCall(_, _) => false,
-            Self::Block(block, _) => block.iter().all(|stmt| stmt.is_pure()),
-            Self::BreakLoop(_, _) | Self::Return(_, _, _) => false,
-            Self::TryCatch(x, _) => {
-                (x.0).0.iter().all(Stmt::is_pure) && (x.2).0.iter().all(Stmt::is_pure)
+            Self::Var(..) | Self::Assignment(..) | Self::FnCall(..) => false,
+            Self::Block(block, ..) => block.iter().all(|stmt| stmt.is_pure()),
+            Self::BreakLoop(..) | Self::Return(..) => false,
+            Self::TryCatch(x, ..) => {
+                x.try_block.iter().all(Stmt::is_pure) && x.catch_block.iter().all(Stmt::is_pure)
             }
 
             #[cfg(not(feature = "no_module"))]
-            Self::Import(_, _, _) => false,
+            Self::Import(..) => false,
             #[cfg(not(feature = "no_module"))]
-            Self::Export(_, _) => false,
+            Self::Export(..) => false,
 
             #[cfg(not(feature = "no_closure"))]
-            Self::Share(_) => false,
+            Self::Share(..) => false,
         }
     }
     /// Does this statement's behavior depend on its containing block?
     ///
-    /// A statement that depends on its containing block behaves differently when promoted
-    /// to an upper block.
+    /// A statement that depends on its containing block behaves differently when promoted to an
+    /// upper block.
     ///
     /// Currently only variable definitions (i.e. `let` and `const`), `import`/`export` statements,
-    /// and `eval` calls (which may in turn call define variables) fall under this category.
+    /// and `eval` calls (which may in turn define variables) fall under this category.
     #[inline]
     #[must_use]
     pub fn is_block_dependent(&self) -> bool {
         match self {
-            Self::Var(_, _, _, _) => true,
+            Self::Var(..) => true,
 
-            Self::Expr(Expr::Stmt(s)) => s.iter().all(Stmt::is_block_dependent),
+            Self::Expr(e) => match &**e {
+                Expr::Stmt(s) => s.iter().all(Stmt::is_block_dependent),
+                Expr::FnCall(x, ..) => !x.is_qualified() && x.name == KEYWORD_EVAL,
+                _ => false,
+            },
 
-            Self::FnCall(x, _) | Self::Expr(Expr::FnCall(x, _)) => {
-                !x.is_qualified() && x.name == KEYWORD_EVAL
-            }
+            Self::FnCall(x, ..) => !x.is_qualified() && x.name == KEYWORD_EVAL,
 
             #[cfg(not(feature = "no_module"))]
-            Self::Import(_, _, _) | Self::Export(_, _) => true,
+            Self::Import(..) | Self::Export(..) => true,
 
             _ => false,
         }
@@ -522,14 +672,17 @@ impl Stmt {
     #[must_use]
     pub fn is_internally_pure(&self) -> bool {
         match self {
-            Self::Var(expr, _, _, _) => expr.is_pure(),
+            Self::Var(x, ..) => x.1.is_pure(),
 
-            Self::Expr(Expr::Stmt(s)) => s.iter().all(Stmt::is_internally_pure),
+            Self::Expr(e) => match e.as_ref() {
+                Expr::Stmt(s) => s.iter().all(Stmt::is_internally_pure),
+                _ => self.is_pure(),
+            },
 
             #[cfg(not(feature = "no_module"))]
-            Self::Import(expr, _, _) => expr.is_pure(),
+            Self::Import(x, ..) => x.0.is_pure(),
             #[cfg(not(feature = "no_module"))]
-            Self::Export(_, _) => true,
+            Self::Export(..) => true,
 
             _ => self.is_pure(),
         }
@@ -543,7 +696,7 @@ impl Stmt {
     #[must_use]
     pub const fn is_control_flow_break(&self) -> bool {
         match self {
-            Self::Return(_, _, _) | Self::BreakLoop(_, _) => true,
+            Self::Return(..) | Self::BreakLoop(..) => true,
             _ => false,
         }
     }
@@ -562,118 +715,133 @@ impl Stmt {
         }
 
         match self {
-            Self::Var(e, _, _, _) => {
-                if !e.walk(path, on_node) {
+            Self::Var(x, ..) => {
+                if !x.1.walk(path, on_node) {
                     return false;
                 }
             }
-            Self::If(e, x, _) => {
-                if !e.walk(path, on_node) {
-                    return false;
-                }
-                for s in &(x.0).0 {
-                    if !s.walk(path, on_node) {
-                        return false;
-                    }
-                }
-                for s in &(x.1).0 {
-                    if !s.walk(path, on_node) {
-                        return false;
-                    }
-                }
-            }
-            Self::Switch(e, x, _) => {
-                if !e.walk(path, on_node) {
-                    return false;
-                }
-                for b in x.0.values() {
-                    if !b.0.as_ref().map(|e| e.walk(path, on_node)).unwrap_or(true) {
-                        return false;
-                    }
-                    for s in &(b.1).0 {
-                        if !s.walk(path, on_node) {
-                            return false;
-                        }
-                    }
-                }
-                for (_, _, _, c, stmt) in &x.2 {
-                    if !c.as_ref().map(|e| e.walk(path, on_node)).unwrap_or(true) {
-                        return false;
-                    }
-                    for s in &stmt.0 {
-                        if !s.walk(path, on_node) {
-                            return false;
-                        }
-                    }
-                }
-                for s in &(x.1).0 {
-                    if !s.walk(path, on_node) {
-                        return false;
-                    }
-                }
-            }
-            Self::While(e, s, _) | Self::Do(s, e, _, _) => {
-                if !e.walk(path, on_node) {
-                    return false;
-                }
-                for s in &s.0 {
-                    if !s.walk(path, on_node) {
-                        return false;
-                    }
-                }
-            }
-            Self::For(e, x, _) => {
-                if !e.walk(path, on_node) {
-                    return false;
-                }
-                for s in &(x.2).0 {
-                    if !s.walk(path, on_node) {
-                        return false;
-                    }
-                }
-            }
-            Self::Assignment(x, _) => {
+            Self::If(x, ..) => {
                 if !x.0.walk(path, on_node) {
                     return false;
                 }
+                for s in x.1.iter() {
+                    if !s.walk(path, on_node) {
+                        return false;
+                    }
+                }
+                for s in x.2.iter() {
+                    if !s.walk(path, on_node) {
+                        return false;
+                    }
+                }
+            }
+            Self::Switch(x, ..) => {
+                if !x.0.walk(path, on_node) {
+                    return false;
+                }
+                for b in x.1.cases.values() {
+                    if !b
+                        .condition
+                        .as_ref()
+                        .map(|e| e.walk(path, on_node))
+                        .unwrap_or(true)
+                    {
+                        return false;
+                    }
+                    for s in b.statements.iter() {
+                        if !s.walk(path, on_node) {
+                            return false;
+                        }
+                    }
+                }
+                for (.., b) in &x.1.ranges {
+                    if !b
+                        .condition
+                        .as_ref()
+                        .map(|e| e.walk(path, on_node))
+                        .unwrap_or(true)
+                    {
+                        return false;
+                    }
+                    for s in b.statements.iter() {
+                        if !s.walk(path, on_node) {
+                            return false;
+                        }
+                    }
+                }
+                for s in x.1.def_case.iter() {
+                    if !s.walk(path, on_node) {
+                        return false;
+                    }
+                }
+            }
+            Self::While(x, ..) | Self::Do(x, ..) => {
+                if !x.0.walk(path, on_node) {
+                    return false;
+                }
+                for s in x.1.statements() {
+                    if !s.walk(path, on_node) {
+                        return false;
+                    }
+                }
+            }
+            Self::For(x, ..) => {
                 if !x.2.walk(path, on_node) {
                     return false;
                 }
+                for s in x.3.iter() {
+                    if !s.walk(path, on_node) {
+                        return false;
+                    }
+                }
             }
-            Self::FnCall(x, _) => {
+            Self::Assignment(x, ..) => {
+                if !x.1.lhs.walk(path, on_node) {
+                    return false;
+                }
+                if !x.1.rhs.walk(path, on_node) {
+                    return false;
+                }
+            }
+            Self::FnCall(x, ..) => {
                 for s in &x.args {
                     if !s.walk(path, on_node) {
                         return false;
                     }
                 }
             }
-            Self::Block(x, _) => {
-                for s in x.iter() {
+            Self::Block(x, ..) => {
+                for s in x.statements() {
                     if !s.walk(path, on_node) {
                         return false;
                     }
                 }
             }
-            Self::TryCatch(x, _) => {
-                for s in &(x.0).0 {
+            Self::TryCatch(x, ..) => {
+                for s in x.try_block.iter() {
                     if !s.walk(path, on_node) {
                         return false;
                     }
                 }
-                for s in &(x.2).0 {
+                for s in x.catch_block.iter() {
                     if !s.walk(path, on_node) {
                         return false;
                     }
                 }
             }
-            Self::Expr(e) | Self::Return(_, Some(e), _) => {
+            Self::Expr(e) => {
+                if !e.walk(path, on_node) {
+                    return false;
+                }
+            }
+            Self::Return(Some(e), ..) => {
                 if !e.walk(path, on_node) {
                     return false;
                 }
             }
             #[cfg(not(feature = "no_module"))]
-            Self::Import(e, _, _) => {
-                if !e.walk(path, on_node) {
+            Self::Import(x, ..) => {
+                if !x.0.walk(path, on_node) {
                     return false;
                 }
             }
