@@ -3,26 +3,26 @@
 use super::{EvalContext, EvalState, GlobalRuntimeState, Target};
 use crate::ast::{Expr, FnCallExpr, OpAssignment};
 use crate::engine::{KEYWORD_THIS, OP_CONCAT};
-use crate::module::Namespace;
 use crate::types::dynamic::AccessMode;
-use crate::{
-    Dynamic, Engine, Module, Position, RhaiResult, RhaiResultOf, Scope, Shared, StaticVec, ERR,
-};
+use crate::{Dynamic, Engine, Module, Position, RhaiResult, RhaiResultOf, Scope, StaticVec, ERR};
 use std::num::NonZeroUsize;
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 
 impl Engine {
     /// Search for a module within an imports stack.
+    #[cfg(not(feature = "no_module"))]
     #[inline]
     #[must_use]
     pub(crate) fn search_imports(
         &self,
         global: &GlobalRuntimeState,
         state: &mut EvalState,
-        namespace: &Namespace,
-    ) -> Option<Shared<Module>> {
-        let root = &namespace[0].name;
+        namespace: &crate::ast::Namespace,
+    ) -> Option<crate::Shared<Module>> {
+        assert!(!namespace.is_empty());
+
+        let root = namespace.root();
 
         // Qualified - check if the root module is directly indexed
         let index = if state.always_search_scope {
@@ -52,50 +52,50 @@ impl Engine {
         lib: &[&Module],
         this_ptr: &'s mut Option<&mut Dynamic>,
         expr: &Expr,
+        level: usize,
     ) -> RhaiResultOf<(Target<'s>, Position)> {
         match expr {
-            Expr::Variable(Some(_), _, _) => {
-                self.search_scope_only(scope, global, state, lib, this_ptr, expr)
+            Expr::Variable(_, Some(_), _) => {
+                self.search_scope_only(scope, global, state, lib, this_ptr, expr, level)
             }
-            Expr::Variable(None, _var_pos, v) => match v.as_ref() {
+            Expr::Variable(v, None, _var_pos) => match v.as_ref() {
                 // Normal variable access
-                (_, None, _) => self.search_scope_only(scope, global, state, lib, this_ptr, expr),
+                #[cfg(not(feature = "no_module"))]
+                (_, ns, ..) if ns.is_empty() => {
+                    self.search_scope_only(scope, global, state, lib, this_ptr, expr, level)
+                }
+                #[cfg(feature = "no_module")]
+                (_, (), ..) => {
+                    self.search_scope_only(scope, global, state, lib, this_ptr, expr, level)
+                }
 
                 // Qualified variable access
                 #[cfg(not(feature = "no_module"))]
-                (_, Some((namespace, hash_var)), var_name) => {
+                (_, namespace, hash_var, var_name) => {
+                    // foo:bar::baz::VARIABLE
                     if let Some(module) = self.search_imports(global, state, namespace) {
-                        // foo:bar::baz::VARIABLE
-                        return match module.get_qualified_var(*hash_var) {
-                            Ok(target) => {
-                                let mut target = target.clone();
-                                // Module variables are constant
-                                target.set_access_mode(AccessMode::ReadOnly);
-                                Ok((target.into(), *_var_pos))
-                            }
-                            Err(err) => Err(match *err {
-                                ERR::ErrorVariableNotFound(_, _) => ERR::ErrorVariableNotFound(
-                                    format!(
-                                        "{}{}{}",
-                                        namespace,
-                                        crate::tokenizer::Token::DoubleColon.literal_syntax(),
-                                        var_name
-                                    ),
-                                    namespace[0].pos,
-                                )
-                                .into(),
-                                _ => err.fill_position(*_var_pos),
-                            }),
+                        return if let Some(mut target) = module.get_qualified_var(*hash_var) {
+                            // Module variables are constant
+                            target.set_access_mode(AccessMode::ReadOnly);
+                            Ok((target.into(), *_var_pos))
+                        } else {
+                            let sep = crate::tokenizer::Token::DoubleColon.literal_syntax();
+
+                            Err(ERR::ErrorVariableNotFound(
+                                format!("{}{}{}", namespace, sep, var_name),
+                                namespace.position(),
+                            )
+                            .into())
                         };
                     }
 
+                    // global::VARIABLE
                     #[cfg(not(feature = "no_function"))]
-                    if namespace.len() == 1 && namespace[0].name == crate::engine::KEYWORD_GLOBAL {
-                        // global::VARIABLE
-                        let global_constants = global.constants_mut();
-
-                        if let Some(mut guard) = global_constants {
-                            if let Some(value) = guard.get_mut(var_name) {
+                    if namespace.len() == 1 && namespace.root() == crate::engine::KEYWORD_GLOBAL {
+                        if let Some(ref constants) = global.constants {
+                            if let Some(value) =
+                                crate::func::locked_write(constants).get_mut(var_name)
+                            {
                                 let mut target: Target = value.clone().into();
                                 // Module variables are constant
                                 target.set_access_mode(AccessMode::ReadOnly);
@@ -103,23 +103,20 @@ impl Engine {
                             }
                         }
 
+                        let sep = crate::tokenizer::Token::DoubleColon.literal_syntax();
+
                         return Err(ERR::ErrorVariableNotFound(
-                            format!(
-                                "{}{}{}",
-                                namespace,
-                                crate::tokenizer::Token::DoubleColon.literal_syntax(),
-                                var_name
-                            ),
-                            namespace[0].pos,
+                            format!("{}{}{}", namespace, sep, var_name),
+                            namespace.position(),
                         )
                         .into());
                     }
 
-                    Err(ERR::ErrorModuleNotFound(namespace.to_string(), namespace[0].pos).into())
+                    Err(
+                        ERR::ErrorModuleNotFound(namespace.to_string(), namespace.position())
+                            .into(),
+                    )
                 }
-
-                #[cfg(feature = "no_module")]
-                (_, Some((_, _)), _) => unreachable!("qualified access under no_module"),
             },
             _ => unreachable!("Expr::Variable expected but gets {:?}", expr),
         }
@@ -138,21 +135,22 @@ impl Engine {
         lib: &[&Module],
         this_ptr: &'s mut Option<&mut Dynamic>,
         expr: &Expr,
+        level: usize,
     ) -> RhaiResultOf<(Target<'s>, Position)> {
         // Make sure that the pointer indirection is taken only when absolutely necessary.
 
         let (index, var_pos) = match expr {
             // Check if the variable is `this`
-            Expr::Variable(None, pos, v) if v.0.is_none() && v.2 == KEYWORD_THIS => {
+            Expr::Variable(v, None, pos) if v.0.is_none() && v.3 == KEYWORD_THIS => {
                 return if let Some(val) = this_ptr {
                     Ok(((*val).into(), *pos))
                 } else {
                     Err(ERR::ErrorUnboundThis(*pos).into())
                 }
             }
-            _ if state.always_search_scope => (0, expr.position()),
-            Expr::Variable(Some(i), pos, _) => (i.get() as usize, *pos),
-            Expr::Variable(None, pos, v) => (v.0.map(NonZeroUsize::get).unwrap_or(0), *pos),
+            _ if state.always_search_scope => (0, expr.start_position()),
+            Expr::Variable(.., Some(i), pos) => (i.get() as usize, *pos),
+            Expr::Variable(v, None, pos) => (v.0.map(NonZeroUsize::get).unwrap_or(0), *pos),
             _ => unreachable!("Expr::Variable expected but gets {:?}", expr),
         };
 
@@ -165,13 +163,10 @@ impl Engine {
                 state,
                 lib,
                 this_ptr,
-                level: 0,
+                level,
             };
-            match resolve_var(
-                expr.get_variable_name(true).expect("`Expr::Variable`"),
-                index,
-                &context,
-            ) {
+            let var_name = expr.get_variable_name(true).expect("`Expr::Variable`");
+            match resolve_var(var_name, index, &context) {
                 Ok(Some(mut result)) => {
                     result.set_access_mode(AccessMode::ReadOnly);
                     return Ok((result.into(), var_pos));
@@ -186,10 +181,18 @@ impl Engine {
         } else {
             // Find the variable in the scope
             let var_name = expr.get_variable_name(true).expect("`Expr::Variable`");
-            scope
-                .get_index(var_name)
-                .ok_or_else(|| ERR::ErrorVariableNotFound(var_name.to_string(), var_pos))?
-                .0
+
+            match scope.get_index(var_name) {
+                Some((index, _)) => index,
+                None => {
+                    return match self.global_modules.iter().find_map(|m| m.get_var(var_name)) {
+                        Some(val) => Ok((val.into(), var_pos)),
+                        None => {
+                            Err(ERR::ErrorVariableNotFound(var_name.to_string(), var_pos).into())
+                        }
+                    }
+                }
+            }
         };
 
         let val = scope.get_mut_by_index(index);
@@ -211,37 +214,44 @@ impl Engine {
     ) -> RhaiResult {
         let FnCallExpr {
             name,
+            #[cfg(not(feature = "no_module"))]
             namespace,
             capture_parent_scope: capture,
             hashes,
             args,
-            constants,
             ..
         } = expr;
 
-        if let Some(namespace) = namespace.as_ref() {
+        #[cfg(not(feature = "no_module"))]
+        if !namespace.is_empty() {
             // Qualified function call
             let hash = hashes.native;
 
-            self.make_qualified_function_call(
-                scope, global, state, lib, this_ptr, namespace, name, args, constants, hash, pos,
-                level,
-            )
-        } else {
-            // Normal function call
-            let (first_arg, args) = args.split_first().map_or_else(
-                || (None, args.as_ref()),
-                |(first, rest)| (Some(first), rest),
+            return self.make_qualified_function_call(
+                scope, global, state, lib, this_ptr, namespace, name, args, hash, pos, level,
             );
-
-            self.make_function_call(
-                scope, global, state, lib, this_ptr, name, first_arg, args, constants, *hashes,
-                pos, *capture, level,
-            )
         }
+
+        // Normal function call
+        let (first_arg, args) = args.split_first().map_or_else(
+            || (None, args.as_ref()),
+            |(first, rest)| (Some(first), rest),
+        );
+
+        self.make_function_call(
+            scope, global, state, lib, this_ptr, name, first_arg, args, *hashes, *capture, pos,
+            level,
+        )
     }
 
     /// Evaluate an expression.
+    //
+    // # Implementation Notes
+    //
+    // Do not use the `?` operator within the main body as it makes this function return early,
+    // possibly by-passing important cleanup tasks at the end.
+    //
+    // Errors that are not recoverable, such as system errors or safety errors, can use `?`.
     pub(crate) fn eval_expr(
         &self,
         scope: &mut Scope,
@@ -257,89 +267,119 @@ impl Engine {
 
         // Function calls should account for a relatively larger portion of expressions because
         // binary operators are also function calls.
-        if let Expr::FnCall(x, pos) = expr {
+        if let Expr::FnCall(x, ..) = expr {
+            #[cfg(feature = "debugging")]
+            let reset_debugger =
+                self.run_debugger_with_reset(scope, global, state, lib, this_ptr, expr, level)?;
+
             #[cfg(not(feature = "unchecked"))]
             self.inc_operations(&mut global.num_operations, expr.position())?;
 
-            return self.eval_fn_call_expr(scope, global, state, lib, this_ptr, x, *pos, level);
+            let result =
+                self.eval_fn_call_expr(scope, global, state, lib, this_ptr, x, x.pos, level);
+
+            #[cfg(feature = "debugging")]
+            global.debugger.reset_status(reset_debugger);
+
+            return result;
         }
 
         // Then variable access.
         // We shouldn't do this for too many variants because, soon or later, the added comparisons
         // will cost more than the mis-predicted `match` branch.
-        if let Expr::Variable(index, var_pos, x) = expr {
+        if let Expr::Variable(x, index, var_pos) = expr {
+            #[cfg(feature = "debugging")]
+            self.run_debugger(scope, global, state, lib, this_ptr, expr, level)?;
+
             #[cfg(not(feature = "unchecked"))]
             self.inc_operations(&mut global.num_operations, expr.position())?;
 
-            return if index.is_none() && x.0.is_none() && x.2 == KEYWORD_THIS {
+            return if index.is_none() && x.0.is_none() && x.3 == KEYWORD_THIS {
                 this_ptr
                     .as_deref()
                     .cloned()
                     .ok_or_else(|| ERR::ErrorUnboundThis(*var_pos).into())
             } else {
-                self.search_namespace(scope, global, state, lib, this_ptr, expr)
-                    .map(|(val, _)| val.take_or_clone())
+                self.search_namespace(scope, global, state, lib, this_ptr, expr, level)
+                    .map(|(val, ..)| val.take_or_clone())
             };
         }
 
+        #[cfg(feature = "debugging")]
+        let reset_debugger =
+            self.run_debugger_with_reset(scope, global, state, lib, this_ptr, expr, level)?;
+
         #[cfg(not(feature = "unchecked"))]
         self.inc_operations(&mut global.num_operations, expr.position())?;
 
-        match expr {
+        let result = match expr {
             // Constants
-            Expr::DynamicConstant(x, _) => Ok(x.as_ref().clone()),
-            Expr::IntegerConstant(x, _) => Ok((*x).into()),
+            Expr::DynamicConstant(x, ..) => Ok(x.as_ref().clone()),
+            Expr::IntegerConstant(x, ..) => Ok((*x).into()),
             #[cfg(not(feature = "no_float"))]
-            Expr::FloatConstant(x, _) => Ok((*x).into()),
-            Expr::StringConstant(x, _) => Ok(x.clone().into()),
-            Expr::CharConstant(x, _) => Ok((*x).into()),
-            Expr::BoolConstant(x, _) => Ok((*x).into()),
-            Expr::Unit(_) => Ok(Dynamic::UNIT),
+            Expr::FloatConstant(x, ..) => Ok((*x).into()),
+            Expr::StringConstant(x, ..) => Ok(x.clone().into()),
+            Expr::CharConstant(x, ..) => Ok((*x).into()),
+            Expr::BoolConstant(x, ..) => Ok((*x).into()),
+            Expr::Unit(..) => Ok(Dynamic::UNIT),
 
             // `... ${...} ...`
-            Expr::InterpolatedString(x, pos) => {
-                let mut pos = *pos;
-                let mut result: Dynamic = self.const_empty_string().into();
+            Expr::InterpolatedString(x, _) => {
+                let mut concat: Dynamic = self.const_empty_string().into();
+                let mut result = Ok(Dynamic::UNIT);
 
                 for expr in x.iter() {
-                    let item = self.eval_expr(scope, global, state, lib, this_ptr, expr, level)?;
+                    let item =
+                        match self.eval_expr(scope, global, state, lib, this_ptr, expr, level) {
+                            Ok(r) => r,
+                            err => {
+                                result = err;
+                                break;
+                            }
+                        };
 
-                    self.eval_op_assignment(
+                    if let Err(err) = self.eval_op_assignment(
                         global,
                         state,
                         lib,
                         Some(OpAssignment::new(OP_CONCAT)),
-                        pos,
-                        &mut (&mut result).into(),
+                        expr.start_position(),
+                        &mut (&mut concat).into(),
                         ("", Position::NONE),
                         item,
-                    )
-                    .map_err(|err| err.fill_position(expr.position()))?;
-
-                    pos = expr.position();
+                        level,
+                    ) {
+                        result = Err(err.fill_position(expr.start_position()));
+                        break;
+                    }
                 }
 
-                Ok(result)
+                result.map(|_| concat)
             }
 
             #[cfg(not(feature = "no_index"))]
-            Expr::Array(x, _) => {
-                let mut arr = Dynamic::from_array(crate::Array::with_capacity(x.len()));
+            Expr::Array(x, ..) => {
+                let mut arr = crate::Array::with_capacity(x.len());
+                let mut result = Ok(Dynamic::UNIT);
 
                 #[cfg(not(feature = "unchecked"))]
                 let mut sizes = (0, 0, 0);
 
                 for item_expr in x.iter() {
-                    let value = self
-                        .eval_expr(scope, global, state, lib, this_ptr, item_expr, level)?
-                        .flatten();
+                    let value = match self
+                        .eval_expr(scope, global, state, lib, this_ptr, item_expr, level)
+                    {
+                        Ok(r) => r.flatten(),
+                        err => {
+                            result = err;
+                            break;
+                        }
+                    };
 
                     #[cfg(not(feature = "unchecked"))]
                     let val_sizes = Self::calc_data_sizes(&value, true);
 
-                    arr.write_lock::<crate::Array>()
-                        .expect("`Array`")
-                        .push(value);
+                    arr.push(value);
 
                     #[cfg(not(feature = "unchecked"))]
                     if self.has_data_size_limit() {
@@ -352,68 +392,87 @@ impl Engine {
                     }
                 }
 
-                Ok(arr)
+                result.map(|_| arr.into())
             }
 
             #[cfg(not(feature = "no_object"))]
-            Expr::Map(x, _) => {
-                let mut map = Dynamic::from_map(x.1.clone());
+            Expr::Map(x, ..) => {
+                let mut map = x.1.clone();
+                let mut result = Ok(Dynamic::UNIT);
 
                 #[cfg(not(feature = "unchecked"))]
                 let mut sizes = (0, 0, 0);
 
-                for (crate::ast::Ident { name, .. }, value_expr) in x.0.iter() {
-                    let key = name.as_str();
-                    let value = self
-                        .eval_expr(scope, global, state, lib, this_ptr, value_expr, level)?
-                        .flatten();
+                for (key, value_expr) in x.0.iter() {
+                    let value = match self
+                        .eval_expr(scope, global, state, lib, this_ptr, value_expr, level)
+                    {
+                        Ok(r) => r.flatten(),
+                        err => {
+                            result = err;
+                            break;
+                        }
+                    };
 
                     #[cfg(not(feature = "unchecked"))]
-                    let val_sizes = Self::calc_data_sizes(&value, true);
+                    let delta = Self::calc_data_sizes(&value, true);
 
-                    *map.write_lock::<crate::Map>()
-                        .expect("`Map`")
-                        .get_mut(key)
-                        .unwrap() = value;
+                    *map.get_mut(key.as_str()).unwrap() = value;
 
                     #[cfg(not(feature = "unchecked"))]
                     if self.has_data_size_limit() {
-                        sizes = (
-                            sizes.0 + val_sizes.0,
-                            sizes.1 + val_sizes.1,
-                            sizes.2 + val_sizes.2,
-                        );
+                        sizes = (sizes.0 + delta.0, sizes.1 + delta.1, sizes.2 + delta.2);
                         self.raise_err_if_over_data_size_limit(sizes, value_expr.position())?;
                     }
                 }
 
-                Ok(map)
+                result.map(|_| map.into())
             }
 
-            Expr::And(x, _) => {
-                Ok((self
-                    .eval_expr(scope, global, state, lib, this_ptr, &x.lhs, level)?
-                    .as_bool()
-                    .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, x.lhs.position()))?
-                    && // Short-circuit using &&
-                self
-                    .eval_expr(scope, global, state, lib, this_ptr, &x.rhs, level)?
-                    .as_bool()
-                    .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, x.rhs.position()))?)
-                .into())
+            Expr::And(x, ..) => {
+                let lhs = self
+                    .eval_expr(scope, global, state, lib, this_ptr, &x.lhs, level)
+                    .and_then(|v| {
+                        v.as_bool().map_err(|typ| {
+                            self.make_type_mismatch_err::<bool>(typ, x.lhs.position())
+                        })
+                    });
+
+                if let Ok(true) = lhs {
+                    self.eval_expr(scope, global, state, lib, this_ptr, &x.rhs, level)
+                        .and_then(|v| {
+                            v.as_bool()
+                                .map_err(|typ| {
+                                    self.make_type_mismatch_err::<bool>(typ, x.rhs.position())
+                                })
+                                .map(Into::into)
+                        })
+                } else {
+                    lhs.map(Into::into)
+                }
             }
 
-            Expr::Or(x, _) => {
-                Ok((self
-                    .eval_expr(scope, global, state, lib, this_ptr, &x.lhs, level)?
-                    .as_bool()
-                    .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, x.lhs.position()))?
-                    || // Short-circuit using ||
-                self
-                    .eval_expr(scope, global, state, lib, this_ptr, &x.rhs, level)?
-                    .as_bool()
-                    .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, x.rhs.position()))?)
-                .into())
+            Expr::Or(x, ..) => {
+                let lhs = self
+                    .eval_expr(scope, global, state, lib, this_ptr, &x.lhs, level)
+                    .and_then(|v| {
+                        v.as_bool().map_err(|typ| {
+                            self.make_type_mismatch_err::<bool>(typ, x.lhs.position())
+                        })
+                    });
+
+                if let Ok(false) = lhs {
+                    self.eval_expr(scope, global, state, lib, this_ptr, &x.rhs, level)
+                        .and_then(|v| {
+                            v.as_bool()
+                                .map_err(|typ| {
+                                    self.make_type_mismatch_err::<bool>(typ, x.rhs.position())
+                                })
+                                .map(Into::into)
+                        })
+                } else {
+                    lhs.map(Into::into)
+                }
             }
 
             Expr::Custom(custom, pos) => {
@@ -440,7 +499,7 @@ impl Engine {
 
                 let result = (custom_def.func)(&mut context, &expressions);
 
-                self.check_return_value(result, expr.position())
+                self.check_return_value(result, expr.start_position())
             }
 
             Expr::Stmt(x) if x.is_empty() => Ok(Dynamic::UNIT),
@@ -449,16 +508,21 @@ impl Engine {
             }
 
             #[cfg(not(feature = "no_index"))]
-            Expr::Index(_, _, _) => {
+            Expr::Index(..) => {
                 self.eval_dot_index_chain(scope, global, state, lib, this_ptr, expr, level, None)
             }
 
             #[cfg(not(feature = "no_object"))]
-            Expr::Dot(_, _, _) => {
+            Expr::Dot(..) => {
                 self.eval_dot_index_chain(scope, global, state, lib, this_ptr, expr, level, None)
             }
 
             _ => unreachable!("expression cannot be evaluated: {:?}", expr),
-        }
+        };
+
+        #[cfg(feature = "debugging")]
+        global.debugger.reset_status(reset_debugger);
+
+        return result;
     }
 }
