@@ -7,7 +7,7 @@ use std::prelude::v1::*;
 
 def_package! {
     /// Package of core language features.
-    crate::LanguageCorePackage => |lib| {
+    pub LanguageCorePackage(lib) {
         lib.standard = true;
 
         combine_with_exported_module!(lib, "core", core_functions);
@@ -21,10 +21,32 @@ def_package! {
 
 #[export_module]
 mod core_functions {
+    /// Return the _tag_ of a `Dynamic` value.
+    ///
+    /// # Example
+    ///
+    /// ```rhai
+    /// let x = "hello, world!";
+    ///
+    /// x.tag = 42;
+    ///
+    /// print(x.tag);           // prints 42
+    /// ```
     #[rhai_fn(name = "tag", get = "tag", pure)]
     pub fn get_tag(value: &mut Dynamic) -> INT {
         value.tag() as INT
     }
+    /// Set the _tag_ of a `Dynamic` value.
+    ///
+    /// # Example
+    ///
+    /// ```rhai
+    /// let x = "hello, world!";
+    ///
+    /// x.tag = 42;
+    ///
+    /// print(x.tag);           // prints 42
+    /// ```
     #[rhai_fn(name = "set_tag", set = "tag", return_raw)]
     pub fn set_tag(value: &mut Dynamic, tag: INT) -> RhaiResultOf<()> {
         if tag < Tag::MIN as INT {
@@ -54,6 +76,30 @@ mod core_functions {
             Ok(())
         }
     }
+
+    /// Block the current thread for a particular number of `seconds`.
+    #[cfg(not(feature = "no_float"))]
+    #[cfg(not(feature = "no_std"))]
+    #[rhai_fn(name = "sleep")]
+    pub fn sleep_float(seconds: crate::FLOAT) {
+        if seconds <= 0.0 {
+            return;
+        }
+
+        #[cfg(not(feature = "f32_float"))]
+        std::thread::sleep(std::time::Duration::from_secs_f64(seconds));
+        #[cfg(feature = "f32_float")]
+        std::thread::sleep(std::time::Duration::from_secs_f32(seconds));
+    }
+
+    /// Block the current thread for a particular number of `seconds`.
+    #[cfg(not(feature = "no_std"))]
+    pub fn sleep(seconds: INT) {
+        if seconds <= 0 {
+            return;
+        }
+        std::thread::sleep(std::time::Duration::from_secs(seconds as u64));
+    }
 }
 
 #[cfg(not(feature = "no_function"))]
@@ -92,15 +138,16 @@ fn collect_fn_metadata(
     // Create a metadata record for a function.
     fn make_metadata(
         dict: &BTreeSet<Identifier>,
-        namespace: Option<Identifier>,
+        #[cfg(not(feature = "no_module"))] namespace: Identifier,
         func: &ScriptFnDef,
     ) -> Map {
         const DICT: &str = "key exists";
 
         let mut map = Map::new();
 
-        if let Some(ns) = namespace {
-            map.insert(dict.get("namespace").expect(DICT).clone(), ns.into());
+        #[cfg(not(feature = "no_module"))]
+        if !namespace.is_empty() {
+            map.insert(dict.get("namespace").expect(DICT).clone(), namespace.into());
         }
         map.insert(
             dict.get("name").expect(DICT).clone(),
@@ -133,6 +180,7 @@ fn collect_fn_metadata(
 
     // Intern strings
     let dict: BTreeSet<Identifier> = [
+        #[cfg(not(feature = "no_module"))]
         "namespace",
         "name",
         "access",
@@ -150,21 +198,52 @@ fn collect_fn_metadata(
     ctx.iter_namespaces()
         .flat_map(Module::iter_script_fn)
         .filter(|(s, a, n, p, f)| filter(*s, *a, n, *p, f))
-        .for_each(|(_, _, _, _, f)| list.push(make_metadata(&dict, None, f).into()));
+        .for_each(|(.., f)| {
+            list.push(
+                make_metadata(
+                    &dict,
+                    #[cfg(not(feature = "no_module"))]
+                    Identifier::new_const(),
+                    f,
+                )
+                .into(),
+            )
+        });
 
     ctx.engine()
         .global_modules
         .iter()
         .flat_map(|m| m.iter_script_fn())
         .filter(|(ns, a, n, p, f)| filter(*ns, *a, n, *p, f))
-        .for_each(|(_, _, _, _, f)| list.push(make_metadata(&dict, None, f).into()));
+        .for_each(|(.., f)| {
+            list.push(
+                make_metadata(
+                    &dict,
+                    #[cfg(not(feature = "no_module"))]
+                    Identifier::new_const(),
+                    f,
+                )
+                .into(),
+            )
+        });
 
+    #[cfg(not(feature = "no_module"))]
     ctx.engine()
         .global_sub_modules
         .values()
         .flat_map(|m| m.iter_script_fn())
         .filter(|(ns, a, n, p, f)| filter(*ns, *a, n, *p, f))
-        .for_each(|(_, _, _, _, f)| list.push(make_metadata(&dict, None, f).into()));
+        .for_each(|(.., f)| {
+            list.push(
+                make_metadata(
+                    &dict,
+                    #[cfg(not(feature = "no_module"))]
+                    Identifier::new_const(),
+                    f,
+                )
+                .into(),
+            )
+        });
 
     #[cfg(not(feature = "no_module"))]
     {
@@ -186,10 +265,8 @@ fn collect_fn_metadata(
             module
                 .iter_script_fn()
                 .filter(|(s, a, n, p, f)| filter(*s, *a, n, *p, f))
-                .for_each(|(_, _, _, _, f)| {
-                    list.push(make_metadata(dict, Some(namespace.clone()), f).into())
-                });
-            module.iter_sub_modules().for_each(|(ns, m)| {
+                .for_each(|(.., f)| list.push(make_metadata(dict, namespace.clone(), f).into()));
+            for (ns, m) in module.iter_sub_modules() {
                 let ns = format!(
                     "{}{}{}",
                     namespace,
@@ -197,11 +274,12 @@ fn collect_fn_metadata(
                     ns
                 );
                 scan_module(list, dict, ns.into(), m.as_ref(), filter)
-            });
+            }
         }
 
-        ctx.iter_imports_raw()
-            .for_each(|(ns, m)| scan_module(&mut list, &dict, ns.clone(), m.as_ref(), filter));
+        for (ns, m) in ctx.iter_imports_raw() {
+            scan_module(&mut list, &dict, ns.clone(), m.as_ref(), filter)
+        }
     }
 
     list
