@@ -13,7 +13,8 @@ def_package! {
     /// * [`BasicStringPackage`][super::BasicStringPackage]
     /// * [`BasicIteratorPackage`][super::BasicIteratorPackage]
     /// * [`BasicFnPackage`][super::BasicFnPackage]
-    crate::CorePackage => |lib| {
+    /// * [`DebuggingPackage`][super::DebuggingPackage]
+    pub CorePackage(lib) {
         lib.standard = true;
 
         super::LanguageCorePackage::init(lib);
@@ -21,5 +22,7 @@ def_package! {
         super::BasicStringPackage::init(lib);
         super::BasicIteratorPackage::init(lib);
         super::BasicFnPackage::init(lib);
+        #[cfg(feature = "debugging")]
+        super::DebuggingPackage::init(lib);
     }
 }
