@@ -5,7 +5,7 @@ use crate::engine::{
     KEYWORD_FN_PTR_CURRY, KEYWORD_IS_DEF_VAR, KEYWORD_PRINT, KEYWORD_THIS, KEYWORD_TYPE_OF,
 };
 use crate::func::native::OnParseTokenCallback;
-use crate::{Engine, LexError, StaticVec, INT, UNSIGNED_INT};
+use crate::{Engine, Identifier, LexError, SmartString, StaticVec, INT, UNSIGNED_INT};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 use std::{
@@ -59,10 +59,10 @@ pub type TokenStream<'a> = Peekable<TokenIterator<'a>>;
 /// Advancing beyond the maximum line length or maximum number of lines is not an error but has no effect.
 #[derive(Eq, PartialEq, Ord, PartialOrd, Hash, Clone, Copy)]
 pub struct Position {
-    /// Line number - 0 = none
+    /// Line number: 0 = none
     #[cfg(not(feature = "no_position"))]
     line: u16,
-    /// Character position - 0 = BOL
+    /// Character position: 0 = BOL
     #[cfg(not(feature = "no_position"))]
     pos: u16,
 }
@@ -94,7 +94,7 @@ impl Position {
     /// Panics if `line` is zero.
     #[inline(always)]
     #[must_use]
-    pub fn new(line: u16, position: u16) -> Self {
+    pub const fn new(line: u16, position: u16) -> Self {
         assert!(line != 0, "line cannot be zero");
 
         let _pos = position;
@@ -106,26 +106,6 @@ impl Position {
             pos: _pos,
         }
     }
-    /// Create a new [`Position`].
-    ///
-    /// If `line` is zero, then [`None`] is returned.
-    ///
-    /// If `position` is zero, then it is at the beginning of a line.
-    #[inline]
-    #[must_use]
-    pub const fn new_const(line: u16, position: u16) -> Option<Self> {
-        if line == 0 {
-            return None;
-        }
-        let _pos = position;
-
-        Some(Self {
-            #[cfg(not(feature = "no_position"))]
-            line,
-            #[cfg(not(feature = "no_position"))]
-            pos: _pos,
-        })
-    }
     /// Get the line number (1-based), or [`None`] if there is no position.
     #[inline]
     #[must_use]
@@ -226,11 +206,9 @@ impl Position {
     /// Print this [`Position`] for debug purposes.
     #[inline]
     pub(crate) fn debug_print(&self, _f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        #[cfg(not(feature = "no_position"))]
         if !self.is_none() {
             write!(_f, " @ {:?}", self)?;
         }
-
         Ok(())
     }
 }
@@ -259,12 +237,19 @@ impl fmt::Display for Position {
 
 impl fmt::Debug for Position {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        #[cfg(not(feature = "no_position"))]
-        write!(f, "{}:{}", self.line, self.pos)?;
-        #[cfg(feature = "no_position")]
-        f.write_str("none")?;
+        if self.is_none() {
+            f.write_str("none")
+        } else {
+            #[cfg(not(feature = "no_position"))]
+            if self.is_beginning_of_line() {
+                write!(f, "{}", self.line)
+            } else {
+                write!(f, "{}:{}", self.line, self.pos)
+            }
 
-        Ok(())
+            #[cfg(feature = "no_position")]
+            unreachable!();
+        }
     }
 }
 
@@ -296,6 +281,71 @@ impl AddAssign for Position {
     }
 }
 
+/// _(internals)_ A span consisting of a starting and an ending [positions][Position].
+/// Exported under the `internals` feature only.
+#[derive(Eq, PartialEq, Ord, PartialOrd, Hash, Clone, Copy, Default)]
+pub struct Span {
+    /// Starting [position][Position].
+    start: Position,
+    /// Ending [position][Position].
+    end: Position,
+}
+
+impl Span {
+    pub const NONE: Self = Self::new(Position::NONE, Position::NONE);
+
+    /// Create a new [`Span`].
+    #[inline(always)]
+    #[must_use]
+    pub const fn new(start: Position, end: Position) -> Self {
+        Self { start, end }
+    }
+    /// Is this [`Span`] non-existent?
+    #[inline(always)]
+    #[must_use]
+    pub const fn is_none(&self) -> bool {
+        self.start.is_none() && self.end.is_none()
+    }
+    /// Get the [`Span`]'s starting [position][Position].
+    #[inline(always)]
+    #[must_use]
+    pub const fn start(&self) -> Position {
+        self.start
+    }
+    /// Get the [`Span`]'s ending [position][Position].
+    #[inline(always)]
+    #[must_use]
+    pub const fn end(&self) -> Position {
+        self.end
+    }
+}
+
+impl fmt::Display for Span {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match (self.start().is_none(), self.end().is_none()) {
+            (false, false) if self.start().line() != self.end().line() => {
+                write!(f, "{:?}-{:?}", self.start(), self.end())
+            }
+            (false, false) => write!(
+                f,
+                "{}:{}-{}",
+                self.start().line().unwrap(),
+                self.start().position().unwrap_or(0),
+                self.end().position().unwrap_or(0)
+            ),
+            (true, false) => write!(f, "..{:?}", self.end()),
+            (false, true) => write!(f, "{:?}", self.start()),
+            (true, true) => write!(f, "{:?}", Position::NONE),
+        }
+    }
+}
+
+impl fmt::Debug for Span {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Display::fmt(self, f)
+    }
+}
+
 /// _(internals)_ A Rhai language token.
 /// Exported under the `internals` feature only.
 #[derive(Debug, PartialEq, Clone, Hash)]
@@ -313,13 +363,13 @@ pub enum Token {
     #[cfg(feature = "decimal")]
     DecimalConstant(rust_decimal::Decimal),
     /// An identifier.
-    Identifier(Box<str>),
+    Identifier(Identifier),
     /// A character constant.
     CharConstant(char),
     /// A string constant.
-    StringConstant(Box<str>),
+    StringConstant(SmartString),
     /// An interpolated string.
-    InterpolatedString(Box<str>),
+    InterpolatedString(SmartString),
     /// `{`
     LeftBrace,
     /// `}`
@@ -484,13 +534,13 @@ pub enum Token {
     #[cfg(not(feature = "no_module"))]
     As,
     /// A lexer error.
-    LexError(LexError),
+    LexError(Box<LexError>),
     /// A comment block.
-    Comment(Box<str>),
+    Comment(SmartString),
     /// A reserved symbol.
-    Reserved(Box<str>),
+    Reserved(SmartString),
     /// A custom keyword.
-    Custom(Box<str>),
+    Custom(SmartString),
     /// End of the input stream.
     EOF,
 }
@@ -599,8 +649,8 @@ impl Token {
             FloatConstant(f) => f.to_string().into(),
             #[cfg(feature = "decimal")]
             DecimalConstant(d) => d.to_string().into(),
-            StringConstant(_) => "string".into(),
-            InterpolatedString(_) => "string".into(),
+            StringConstant(..) => "string".into(),
+            InterpolatedString(..) => "string".into(),
             CharConstant(c) => c.to_string().into(),
             Identifier(s) => s.to_string().into(),
             Reserved(s) => s.to_string().into(),
@@ -821,7 +871,7 @@ impl Token {
         use Token::*;
 
         match self {
-            LexError(_)      |
+            LexError(..)      |
             SemiColon        | // ; - is unary
             Colon            | // #{ foo: - is unary
             Comma            | // ( ... , -expr ) - is unary
@@ -966,15 +1016,15 @@ impl Token {
     #[inline(always)]
     #[must_use]
     pub const fn is_reserved(&self) -> bool {
-        matches!(self, Self::Reserved(_))
+        matches!(self, Self::Reserved(..))
     }
 
     /// Convert a token into a function name, if possible.
     #[cfg(not(feature = "no_function"))]
     #[inline]
-    pub(crate) fn into_function_name_for_override(self) -> Result<Box<str>, Self> {
+    pub(crate) fn into_function_name_for_override(self) -> Result<SmartString, Self> {
         match self {
-            Self::Custom(s) | Self::Identifier(s) if is_valid_function_name(&*s) => Ok(s),
+            Self::Custom(s) | Self::Identifier(s) if is_valid_function_name(&s) => Ok(s),
             _ => Err(self),
         }
     }
@@ -983,7 +1033,7 @@ impl Token {
     #[inline(always)]
     #[must_use]
     pub const fn is_custom(&self) -> bool {
-        matches!(self, Self::Custom(_))
+        matches!(self, Self::Custom(..))
     }
 }
 
@@ -1062,11 +1112,12 @@ pub fn parse_string_literal(
     verbatim: bool,
     allow_line_continuation: bool,
     allow_interpolation: bool,
-) -> Result<(Box<str>, bool), (LexError, Position)> {
-    let mut result = String::with_capacity(12);
-    let mut escape = String::with_capacity(12);
+) -> Result<(SmartString, bool, Position), (LexError, Position)> {
+    let mut result = SmartString::new_const();
+    let mut escape = SmartString::new_const();
 
     let start = *pos;
+    let mut first_char = Position::NONE;
     let mut interpolated = false;
     #[cfg(not(feature = "no_position"))]
     let mut skip_whitespace_until = 0;
@@ -1095,7 +1146,6 @@ pub fn parse_string_literal(
                 break;
             }
             None => {
-                result += &escape;
                 pos.advance();
                 state.is_within_text_terminated_by = None;
                 return Err((LERR::UnterminatedString, start));
@@ -1119,6 +1169,21 @@ pub fn parse_string_literal(
             }
         }
 
+        // Close wrapper
+        if termination_char == next_char && escape.is_empty() {
+            // Double wrapper
+            if stream.peek_next().map_or(false, |c| c == termination_char) {
+                eat_next(stream, pos);
+            } else {
+                state.is_within_text_terminated_by = None;
+                break;
+            }
+        }
+
+        if first_char.is_none() {
+            first_char = *pos;
+        }
+
         match next_char {
             // \r - ignore if followed by \n
             '\r' if stream.peek_next().map(|ch| ch == '\n').unwrap_or(false) => (),
@@ -1176,7 +1241,7 @@ pub fn parse_string_literal(
 
                 result.push(
                     char::from_u32(out_val)
-                        .ok_or_else(|| (LERR::MalformedEscapeSequence(seq), *pos))?,
+                        .ok_or_else(|| (LERR::MalformedEscapeSequence(seq.to_string()), *pos))?,
                 );
             }
 
@@ -1186,21 +1251,6 @@ pub fn parse_string_literal(
                 result.push(next_char)
             }
 
-            // Double wrapper
-            _ if termination_char == next_char
-                && escape.is_empty()
-                && stream.peek_next().map_or(false, |c| c == termination_char) =>
-            {
-                eat_next(stream, pos);
-                result.push(termination_char)
-            }
-
-            // Close wrapper
-            _ if termination_char == next_char && escape.is_empty() => {
-                state.is_within_text_terminated_by = None;
-                break;
-            }
-
             // Verbatim
             '\n' if verbatim => {
                 assert_eq!(escape, "", "verbatim strings should not have any escapes");
@@ -1232,7 +1282,7 @@ pub fn parse_string_literal(
             _ if !escape.is_empty() => {
                 escape.push(next_char);
 
-                return Err((LERR::MalformedEscapeSequence(escape), *pos));
+                return Err((LERR::MalformedEscapeSequence(escape.to_string()), *pos));
             }
 
             // Whitespace to skip
@@ -1258,7 +1308,7 @@ pub fn parse_string_literal(
         }
     }
 
-    Ok((result.into(), interpolated))
+    Ok((result, interpolated, first_char))
 }
 
 /// Consume the next character.
@@ -1273,7 +1323,7 @@ fn scan_block_comment(
     stream: &mut impl InputStream,
     level: usize,
     pos: &mut Position,
-    comment: Option<&mut String>,
+    comment: Option<&mut SmartString>,
 ) -> usize {
     let mut level = level;
     let mut comment = comment;
@@ -1328,7 +1378,7 @@ pub fn get_next_token(
     let result = get_next_token_inner(stream, state, pos);
 
     // Save the last token's state
-    if let Some((ref token, _)) = result {
+    if let Some((ref token, ..)) = result {
         state.next_token_cannot_be_unary = !token.is_next_unary();
     }
 
@@ -1368,7 +1418,7 @@ fn get_next_token_inner(
     if state.comment_level > 0 {
         let start_pos = *pos;
         let mut comment = if state.include_comments {
-            Some(String::new())
+            Some(SmartString::new_const())
         } else {
             None
         };
@@ -1393,11 +1443,9 @@ fn get_next_token_inner(
 
     // Within text?
     if let Some(ch) = state.is_within_text_terminated_by.take() {
-        let start_pos = *pos;
-
         return parse_string_literal(stream, state, pos, ch, true, false, true).map_or_else(
-            |(err, err_pos)| Some((Token::LexError(err), err_pos)),
-            |(result, interpolated)| {
+            |(err, err_pos)| Some((Token::LexError(err.into()), err_pos)),
+            |(result, interpolated, start_pos)| {
                 if interpolated {
                     Some((Token::InterpolatedString(result), start_pos))
                 } else {
@@ -1416,10 +1464,10 @@ fn get_next_token_inner(
 
         match (c, stream.peek_next().unwrap_or('\0')) {
             // \n
-            ('\n', _) => pos.new_line(),
+            ('\n', ..) => pos.new_line(),
 
             // digit ...
-            ('0'..='9', _) => {
+            ('0'..='9', ..) => {
                 let mut result = smallvec::SmallVec::<[char; 16]>::new();
                 let mut radix_base: Option<u32> = None;
                 let mut valid: fn(char) -> bool = is_numeric_digit;
@@ -1534,7 +1582,9 @@ fn get_next_token_inner(
                             .map(|v| v as INT)
                             .map(Token::IntegerConstant)
                             .unwrap_or_else(|_| {
-                                Token::LexError(LERR::MalformedNumber(result.into_iter().collect()))
+                                Token::LexError(
+                                    LERR::MalformedNumber(result.into_iter().collect()).into(),
+                                )
                             })
                     } else {
                         let out: String =
@@ -1560,7 +1610,9 @@ fn get_next_token_inner(
                         });
 
                         num.unwrap_or_else(|_| {
-                            Token::LexError(LERR::MalformedNumber(result.into_iter().collect()))
+                            Token::LexError(
+                                LERR::MalformedNumber(result.into_iter().collect()).into(),
+                            )
                         })
                     },
                     num_pos,
@@ -1569,34 +1621,34 @@ fn get_next_token_inner(
 
             // letter or underscore ...
             #[cfg(not(feature = "unicode-xid-ident"))]
-            ('a'..='z', _) | ('_', _) | ('A'..='Z', _) => {
+            ('a'..='z', ..) | ('_', ..) | ('A'..='Z', ..) => {
                 return get_identifier(stream, pos, start_pos, c);
             }
             #[cfg(feature = "unicode-xid-ident")]
-            (ch, _) if unicode_xid::UnicodeXID::is_xid_start(ch) || ch == '_' => {
+            (ch, ..) if unicode_xid::UnicodeXID::is_xid_start(ch) || ch == '_' => {
                 return get_identifier(stream, pos, start_pos, c);
             }
 
             // " - string literal
-            ('"', _) => {
+            ('"', ..) => {
                 return parse_string_literal(stream, state, pos, c, false, true, false)
                     .map_or_else(
-                        |(err, err_pos)| Some((Token::LexError(err), err_pos)),
-                        |(result, _)| Some((Token::StringConstant(result), start_pos)),
+                        |(err, err_pos)| Some((Token::LexError(err.into()), err_pos)),
+                        |(result, ..)| Some((Token::StringConstant(result), start_pos)),
                     );
             }
             // ` - string literal
-            ('`', _) => {
+            ('`', ..) => {
                 // Start from the next line if at the end of line
                 match stream.peek_next() {
                     // `\r - start from next line
                     Some('\r') => {
                         eat_next(stream, pos);
-                        pos.new_line();
                         // `\r\n
                         if let Some('\n') = stream.peek_next() {
                             eat_next(stream, pos);
                         }
+                        pos.new_line();
                     }
                     // `\n - start from next line
                     Some('\n') => {
@@ -1607,8 +1659,8 @@ fn get_next_token_inner(
                 }
 
                 return parse_string_literal(stream, state, pos, c, true, false, true).map_or_else(
-                    |(err, err_pos)| Some((Token::LexError(err), err_pos)),
-                    |(result, interpolated)| {
+                    |(err, err_pos)| Some((Token::LexError(err.into()), err_pos)),
+                    |(result, interpolated, ..)| {
                         if interpolated {
                             Some((Token::InterpolatedString(result), start_pos))
                         } else {
@@ -1621,21 +1673,21 @@ fn get_next_token_inner(
             // ' - character literal
             ('\'', '\'') => {
                 return Some((
-                    Token::LexError(LERR::MalformedChar("".to_string())),
+                    Token::LexError(LERR::MalformedChar("".to_string()).into()),
                     start_pos,
                 ))
             }
-            ('\'', _) => {
+            ('\'', ..) => {
                 return Some(
                     parse_string_literal(stream, state, pos, c, false, false, false).map_or_else(
-                        |(err, err_pos)| (Token::LexError(err), err_pos),
-                        |(result, _)| {
+                        |(err, err_pos)| (Token::LexError(err.into()), err_pos),
+                        |(result, ..)| {
                             let mut chars = result.chars();
                             let first = chars.next().unwrap();
 
                             if chars.next().is_some() {
                                 (
-                                    Token::LexError(LERR::MalformedChar(result.to_string())),
+                                    Token::LexError(LERR::MalformedChar(result.to_string()).into()),
                                     start_pos,
                                 )
                             } else {
@@ -1647,20 +1699,20 @@ fn get_next_token_inner(
             }
 
             // Braces
-            ('{', _) => return Some((Token::LeftBrace, start_pos)),
-            ('}', _) => return Some((Token::RightBrace, start_pos)),
+            ('{', ..) => return Some((Token::LeftBrace, start_pos)),
+            ('}', ..) => return Some((Token::RightBrace, start_pos)),
 
             // Parentheses
             ('(', '*') => {
                 eat_next(stream, pos);
                 return Some((Token::Reserved("(*".into()), start_pos));
             }
-            ('(', _) => return Some((Token::LeftParen, start_pos)),
-            (')', _) => return Some((Token::RightParen, start_pos)),
+            ('(', ..) => return Some((Token::LeftParen, start_pos)),
+            (')', ..) => return Some((Token::RightParen, start_pos)),
 
             // Indexing
-            ('[', _) => return Some((Token::LeftBracket, start_pos)),
-            (']', _) => return Some((Token::RightBracket, start_pos)),
+            ('[', ..) => return Some((Token::LeftBracket, start_pos)),
+            (']', ..) => return Some((Token::RightBracket, start_pos)),
 
             // Map literal
             #[cfg(not(feature = "no_object"))]
@@ -1682,7 +1734,7 @@ fn get_next_token_inner(
                 return Some((Token::Reserved(token.into()), start_pos));
             }
 
-            ('#', _) => return Some((Token::Reserved("#".into()), start_pos)),
+            ('#', ..) => return Some((Token::Reserved("#".into()), start_pos)),
 
             // Operators
             ('+', '=') => {
@@ -1693,10 +1745,10 @@ fn get_next_token_inner(
                 eat_next(stream, pos);
                 return Some((Token::Reserved("++".into()), start_pos));
             }
-            ('+', _) if !state.next_token_cannot_be_unary => {
+            ('+', ..) if !state.next_token_cannot_be_unary => {
                 return Some((Token::UnaryPlus, start_pos))
             }
-            ('+', _) => return Some((Token::Plus, start_pos)),
+            ('+', ..) => return Some((Token::Plus, start_pos)),
 
             ('-', '0'..='9') if !state.next_token_cannot_be_unary => negated = Some(start_pos),
             ('-', '0'..='9') => return Some((Token::Minus, start_pos)),
@@ -1712,10 +1764,10 @@ fn get_next_token_inner(
                 eat_next(stream, pos);
                 return Some((Token::Reserved("--".into()), start_pos));
             }
-            ('-', _) if !state.next_token_cannot_be_unary => {
+            ('-', ..) if !state.next_token_cannot_be_unary => {
                 return Some((Token::UnaryMinus, start_pos))
             }
-            ('-', _) => return Some((Token::Minus, start_pos)),
+            ('-', ..) => return Some((Token::Minus, start_pos)),
 
             ('*', ')') => {
                 eat_next(stream, pos);
@@ -1738,13 +1790,13 @@ fn get_next_token_inner(
                     start_pos,
                 ));
             }
-            ('*', _) => return Some((Token::Multiply, start_pos)),
+            ('*', ..) => return Some((Token::Multiply, start_pos)),
 
             // Comments
             ('/', '/') => {
                 eat_next(stream, pos);
 
-                let mut comment = match stream.peek_next() {
+                let mut comment: Option<SmartString> = match stream.peek_next() {
                     #[cfg(not(feature = "no_function"))]
                     #[cfg(feature = "metadata")]
                     Some('/') => {
@@ -1753,20 +1805,20 @@ fn get_next_token_inner(
                         // Long streams of `///...` are not doc-comments
                         match stream.peek_next() {
                             Some('/') => None,
-                            _ => Some("///".to_string()),
+                            _ => Some("///".into()),
                         }
                     }
-                    _ if state.include_comments => Some("//".to_string()),
+                    _ if state.include_comments => Some("//".into()),
                     _ => None,
                 };
 
                 while let Some(c) = stream.get_next() {
                     if c == '\r' {
-                        pos.new_line();
                         // \r\n
                         if let Some('\n') = stream.peek_next() {
                             eat_next(stream, pos);
                         }
+                        pos.new_line();
                         break;
                     }
                     if c == '\n' {
@@ -1780,14 +1832,14 @@ fn get_next_token_inner(
                 }
 
                 if let Some(comment) = comment {
-                    return Some((Token::Comment(comment.into()), start_pos));
+                    return Some((Token::Comment(comment), start_pos));
                 }
             }
             ('/', '*') => {
                 state.comment_level = 1;
                 eat_next(stream, pos);
 
-                let mut comment = match stream.peek_next() {
+                let mut comment: Option<SmartString> = match stream.peek_next() {
                     #[cfg(not(feature = "no_function"))]
                     #[cfg(feature = "metadata")]
                     Some('*') => {
@@ -1796,10 +1848,10 @@ fn get_next_token_inner(
                         // Long streams of `/****...` are not doc-comments
                         match stream.peek_next() {
                             Some('*') => None,
-                            _ => Some("/**".to_string()),
+                            _ => Some("/**".into()),
                         }
                     }
-                    _ if state.include_comments => Some("/*".to_string()),
+                    _ if state.include_comments => Some("/*".into()),
                     _ => None,
                 };
 
@@ -1807,7 +1859,7 @@ fn get_next_token_inner(
                     scan_block_comment(stream, state.comment_level, pos, comment.as_mut());
 
                 if let Some(comment) = comment {
-                    return Some((Token::Comment(comment.into()), start_pos));
+                    return Some((Token::Comment(comment), start_pos));
                 }
             }
 
@@ -1815,10 +1867,10 @@ fn get_next_token_inner(
                 eat_next(stream, pos);
                 return Some((Token::DivideAssign, start_pos));
             }
-            ('/', _) => return Some((Token::Divide, start_pos)),
+            ('/', ..) => return Some((Token::Divide, start_pos)),
 
-            (';', _) => return Some((Token::SemiColon, start_pos)),
-            (',', _) => return Some((Token::Comma, start_pos)),
+            (';', ..) => return Some((Token::SemiColon, start_pos)),
+            (',', ..) => return Some((Token::Comma, start_pos)),
 
             ('.', '.') => {
                 eat_next(stream, pos);
@@ -1837,7 +1889,7 @@ fn get_next_token_inner(
                     start_pos,
                 ));
             }
-            ('.', _) => return Some((Token::Period, start_pos)),
+            ('.', ..) => return Some((Token::Period, start_pos)),
 
             ('=', '=') => {
                 eat_next(stream, pos);
@@ -1853,7 +1905,7 @@ fn get_next_token_inner(
                 eat_next(stream, pos);
                 return Some((Token::DoubleArrow, start_pos));
             }
-            ('=', _) => return Some((Token::Equals, start_pos)),
+            ('=', ..) => return Some((Token::Equals, start_pos)),
 
             #[cfg(not(feature = "no_module"))]
             (':', ':') => {
@@ -1874,7 +1926,7 @@ fn get_next_token_inner(
                 eat_next(stream, pos);
                 return Some((Token::Reserved(":;".into()), start_pos));
             }
-            (':', _) => return Some((Token::Colon, start_pos)),
+            (':', ..) => return Some((Token::Colon, start_pos)),
 
             ('<', '=') => {
                 eat_next(stream, pos);
@@ -1897,7 +1949,7 @@ fn get_next_token_inner(
                     start_pos,
                 ));
             }
-            ('<', _) => return Some((Token::LessThan, start_pos)),
+            ('<', ..) => return Some((Token::LessThan, start_pos)),
 
             ('>', '=') => {
                 eat_next(stream, pos);
@@ -1916,7 +1968,7 @@ fn get_next_token_inner(
                     start_pos,
                 ));
             }
-            ('>', _) => return Some((Token::GreaterThan, start_pos)),
+            ('>', ..) => return Some((Token::GreaterThan, start_pos)),
 
             ('!', '=') => {
                 eat_next(stream, pos);
@@ -1928,7 +1980,7 @@ fn get_next_token_inner(
 
                 return Some((Token::NotEqualsTo, start_pos));
             }
-            ('!', _) => return Some((Token::Bang, start_pos)),
+            ('!', ..) => return Some((Token::Bang, start_pos)),
 
             ('|', '|') => {
                 eat_next(stream, pos);
@@ -1938,7 +1990,7 @@ fn get_next_token_inner(
                 eat_next(stream, pos);
                 return Some((Token::OrAssign, start_pos));
             }
-            ('|', _) => return Some((Token::Pipe, start_pos)),
+            ('|', ..) => return Some((Token::Pipe, start_pos)),
 
             ('&', '&') => {
                 eat_next(stream, pos);
@@ -1948,31 +2000,31 @@ fn get_next_token_inner(
                 eat_next(stream, pos);
                 return Some((Token::AndAssign, start_pos));
             }
-            ('&', _) => return Some((Token::Ampersand, start_pos)),
+            ('&', ..) => return Some((Token::Ampersand, start_pos)),
 
             ('^', '=') => {
                 eat_next(stream, pos);
                 return Some((Token::XOrAssign, start_pos));
             }
-            ('^', _) => return Some((Token::XOr, start_pos)),
+            ('^', ..) => return Some((Token::XOr, start_pos)),
 
-            ('~', _) => return Some((Token::Reserved("~".into()), start_pos)),
+            ('~', ..) => return Some((Token::Reserved("~".into()), start_pos)),
 
             ('%', '=') => {
                 eat_next(stream, pos);
                 return Some((Token::ModuloAssign, start_pos));
             }
-            ('%', _) => return Some((Token::Modulo, start_pos)),
+            ('%', ..) => return Some((Token::Modulo, start_pos)),
 
-            ('@', _) => return Some((Token::Reserved("@".into()), start_pos)),
+            ('@', ..) => return Some((Token::Reserved("@".into()), start_pos)),
 
-            ('$', _) => return Some((Token::Reserved("$".into()), start_pos)),
+            ('$', ..) => return Some((Token::Reserved("$".into()), start_pos)),
 
-            (ch, _) if ch.is_whitespace() => (),
+            (ch, ..) if ch.is_whitespace() => (),
 
-            (ch, _) => {
+            (ch, ..) => {
                 return Some((
-                    Token::LexError(LERR::UnexpectedInput(ch.to_string())),
+                    Token::LexError(LERR::UnexpectedInput(ch.to_string()).into()),
                     start_pos,
                 ))
             }
@@ -2014,7 +2066,7 @@ fn get_identifier(
 
     if !is_valid_identifier {
         return Some((
-            Token::LexError(LERR::MalformedIdentifier(identifier)),
+            Token::LexError(LERR::MalformedIdentifier(identifier).into()),
             start_pos,
         ));
     }
@@ -2193,67 +2245,67 @@ impl<'a> Iterator for TokenIterator<'a> {
             // a verbatim string or a string with continuation encounters {EOF}.
             // This is necessary to handle such cases for line-by-line parsing, but for an entire
             // script it is a syntax error.
-            Some((Token::StringConstant(_), pos)) if self.state.is_within_text_terminated_by.is_some() => {
+            Some((Token::StringConstant(..), pos)) if self.state.is_within_text_terminated_by.is_some() => {
                 self.state.is_within_text_terminated_by = None;
-                return Some((Token::LexError(LERR::UnterminatedString), pos));
+                return Some((Token::LexError(LERR::UnterminatedString.into()), pos));
             }
             // Reserved keyword/symbol
             Some((Token::Reserved(s), pos)) => (match
-                (&*s, self.engine.custom_keywords.contains_key(&*s))
+                (&*s, !self.engine.custom_keywords.is_empty() && self.engine.custom_keywords.contains_key(&*s))
             {
                 ("===", false) => Token::LexError(LERR::ImproperSymbol(s.to_string(),
                     "'===' is not a valid operator. This is not JavaScript! Should it be '=='?".to_string(),
-                )),
+                ).into()),
                 ("!==", false) => Token::LexError(LERR::ImproperSymbol(s.to_string(),
                     "'!==' is not a valid operator. This is not JavaScript! Should it be '!='?".to_string(),
-                )),
+                ).into()),
                 ("->", false) => Token::LexError(LERR::ImproperSymbol(s.to_string(),
-                    "'->' is not a valid symbol. This is not C or C++!".to_string())),
+                    "'->' is not a valid symbol. This is not C or C++!".to_string()).into()),
                 ("<-", false) => Token::LexError(LERR::ImproperSymbol(s.to_string(),
                     "'<-' is not a valid symbol. This is not Go! Should it be '<='?".to_string(),
-                )),
+                ).into()),
                 (":=", false) => Token::LexError(LERR::ImproperSymbol(s.to_string(),
                     "':=' is not a valid assignment operator. This is not Go or Pascal! Should it be simply '='?".to_string(),
-                )),
+                ).into()),
                 (":;", false) => Token::LexError(LERR::ImproperSymbol(s.to_string(),
                     "':;' is not a valid symbol. Should it be '::'?".to_string(),
-                )),
+                ).into()),
                 ("::<", false) => Token::LexError(LERR::ImproperSymbol(s.to_string(),
                     "'::<>' is not a valid symbol. This is not Rust! Should it be '::'?".to_string(),
-                )),
+                ).into()),
                 ("(*", false) | ("*)", false) => Token::LexError(LERR::ImproperSymbol(s.to_string(),
                     "'(* .. *)' is not a valid comment format. This is not Pascal! Should it be '/* .. */'?".to_string(),
-                )),
+                ).into()),
                 ("# {", false) => Token::LexError(LERR::ImproperSymbol(s.to_string(),
                     "'#' is not a valid symbol. Should it be '#{'?".to_string(),
-                )),
+                ).into()),
                 // Reserved keyword/operator that is custom.
-                (_, true) => Token::Custom(s),
+                (.., true) => Token::Custom(s),
                 // Reserved keyword that is not custom and disabled.
-                (token, false) if self.engine.disabled_symbols.contains(token) => {
+                (token, false) if !self.engine.disabled_symbols.is_empty() && self.engine.disabled_symbols.contains(token) => {
                     let msg = format!("reserved {} '{}' is disabled", if is_valid_identifier(token.chars()) { "keyword"} else {"symbol"}, token);
-                    Token::LexError(LERR::ImproperSymbol(s.to_string(), msg))
+                    Token::LexError(LERR::ImproperSymbol(s.to_string(), msg).into())
                 },
                 // Reserved keyword/operator that is not custom.
-                (_, false) => Token::Reserved(s),
+                (.., false) => Token::Reserved(s),
             }, pos),
             // Custom keyword
-            Some((Token::Identifier(s), pos)) if self.engine.custom_keywords.contains_key(&*s) => {
+            Some((Token::Identifier(s), pos)) if !self.engine.custom_keywords.is_empty() && self.engine.custom_keywords.contains_key(&*s) => {
                 (Token::Custom(s), pos)
             }
             // Custom keyword/symbol - must be disabled
-            Some((token, pos)) if self.engine.custom_keywords.contains_key(&*token.syntax()) => {
-                if self.engine.disabled_symbols.contains(&*token.syntax()) {
+            Some((token, pos)) if !self.engine.custom_keywords.is_empty() && self.engine.custom_keywords.contains_key(token.literal_syntax()) => {
+                if !self.engine.disabled_symbols.is_empty() && self.engine.disabled_symbols.contains(token.literal_syntax()) {
                     // Disabled standard keyword/symbol
-                    (Token::Custom(token.syntax().into()), pos)
+                    (Token::Custom(token.literal_syntax().into()), pos)
                 } else {
                     // Active standard keyword - should never be a custom keyword!
                     unreachable!("{:?} is an active keyword", token)
                 }
             }
             // Disabled symbol
-            Some((token, pos)) if self.engine.disabled_symbols.contains(&*token.syntax()) => {
-                (Token::Reserved(token.syntax().into()), pos)
+            Some((token, pos)) if !self.engine.disabled_symbols.is_empty() && self.engine.disabled_symbols.contains(token.literal_syntax()) => {
+                (Token::Reserved(token.literal_syntax().into()), pos)
             }
             // Normal symbol
             Some(r) => r,
